/* channel.c --
   
   Part of: ztcl
   Contents: channel interface to gzipped files
   Date: Thu Jan  8, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#include "ztclInt.h"
#include <string.h>
#include <errno.h>
#include <stddef.h>
#include <assert.h>
#include <limits.h>


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define CHANNEL_NAME_LENGTH	(16 + TCL_INTEGER_SPACE)
#define CHANNEL_PATTERN		"ztcl%u"


/** ------------------------------------------------------------
 ** Function prototypes.
 ** ----------------------------------------------------------*/

static Tcl_DriverCloseProc	ZchanClose;
static Tcl_DriverInputProc	ZchanInput;
static Tcl_DriverOutputProc	ZchanOutput;
static Tcl_DriverSeekProc	ZchanSeek;
static Tcl_DriverWatchProc	ZchanWatch;
static Tcl_DriverGetHandleProc	ZchanGetHandle;
static Tcl_DriverSetOptionProc	ZchanSetOption;
static Tcl_DriverGetOptionProc	ZchanGetOption;

static Tcl_TimerProc		ZchanNotifyReadable;
static Tcl_TimerProc		ZchanNotifyWritable;

static More_ChannelDriverSetOptionProc SetOptionLevel;
static More_ChannelDriverSetOptionProc SetOptionStrategy;
static More_ChannelDriverGetOptionProc GetOptionLevel;
static More_ChannelDriverGetOptionProc GetOptionStrategy;


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Channel {
  Ztcl_File		token;
  Tcl_Channel		channel;
  Tcl_TimerToken	readTimer;
  Tcl_TimerToken	writeTimer;
} Channel;


/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Tcl_ChannelType Driver = {
  "ztcl_channel",
  TCL_CHANNEL_VERSION_3,
  ZchanClose,
  ZchanInput,
  ZchanOutput,
  ZchanSeek,
  ZchanSetOption,
  ZchanGetOption,
  ZchanWatch,
  ZchanGetHandle,
  NULL, /* Close2 */
  NULL, /* BlockMode */
  NULL, /* Flush */
  NULL, /* Handler */
  NULL /* WideSeek */
};

static Tcl_Mutex	counterMutex = NULL;
static int		channelCounter = 0;

static CONST More_ChannelDriverOption optionTable[] = {
  { "-level",    SetOptionLevel,    GetOptionLevel },
  { "-strategy", SetOptionStrategy, GetOptionStrategy },
  { NULL, NULL, NULL }
};


Tcl_Channel
Ztcl_MakeChannel (Ztcl_File token)
{
  Channel *	data;
  char	 	channelName[CHANNEL_NAME_LENGTH];
  int		openMode;


  Tcl_MutexLock(&counterMutex);
  sprintf(channelName, CHANNEL_PATTERN, ++channelCounter);
  Tcl_MutexUnlock(&counterMutex);

  openMode = (Ztcl_ReadableFile(token)? TCL_READABLE : TCL_WRITABLE);

  data = (Channel *) ckalloc(sizeof(Channel));
  memset(data, '\0', sizeof(Channel));
  data->token   = token;
  data->channel = Tcl_CreateChannel(&Driver, channelName, data, openMode);

  debug("created channel: name \"%s\", pointer \"%p\"",
	 Tcl_GetChannelName(data->channel), data->channel);
  return data->channel;
}

/* ZchanClose --

	Invoked  by  the  generic   layer  to  clean-up  driver  related
	informations when the channel is closed. 

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.

   Results:

	Frees all the resources  associated to the channel.  Must return
	zero if the  operation is successful, or a  non-zero POSIX error
	code  if an  error occurs;  if an  error occurs  and  "errno" is
	false: EIO is used as POSIX error code.

*/

int
ZchanClose (ClientData D, Tcl_Interp *interp)
{
  Channel *	data = (Channel *) D;
  More_Error	error;
  int		errorCode = 0;


  debug("closing channel \"%s\"", Tcl_GetChannelName(data->channel));
  error = Ztcl_Close(data->token);
  if (error)
    {
      errorCode = More_ErrorGetInt(error);
      More_ErrorResult(interp, error);
    }
  else
    {
      ckfree((char *) data);      
    }
  return errorCode;
}

/* ZchanInput --

	Invoked by  the generic layer to  read data from  the device and
	store it in an internal buffer.

   Arguments:

	D -		Pointer to the instance structure.
	bufPtr -	Pointer to the output buffer.
	bufLen -	Number of bytes to be read.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

	Reads  an object  from the  file and  copy its  contents  in the
	output buffer. This operation can  never block: a file is always
	readable.

	  If the file was opened  for writing an error is returned, with
	POSIX code EACCES.

          Must return  a non-negative integer indicating  how many bytes
        were read. In case of error  must return -1.

*/

int
ZchanInput (ClientData D, char *bufPtr, int bufLen, int *errorCodePtr)
{
  Channel *	data = (Channel *) D;
  More_Block	block;
  More_Error	error;


  if (! Ztcl_ReadableFile(data->token))
    {
      *errorCodePtr = EACCES;
      return -1;
    }

  debug("entering input for \"%s\"", Tcl_GetChannelName(data->channel));

  if (Ztcl_Eof(data->token))
    {
      *errorCodePtr = 0;
      return 0;
    }

  block.ptr = bufPtr;
  block.len = bufLen;

  debug("requested reading %d bytes", block.len);

  error = Ztcl_Read(data->token, &block);
  if (! error)
    {
      debug("success");
      *errorCodePtr = 0;
      return block.len;
    }
  else
    {
      *errorCodePtr = More_ErrorGetInt(error);
      More_ErrorForget(error);
      return -1;
    }
}

/* ZchanOutput --

	Invoked by the  generic layer to transfer data  from an internal
	buffer to the output device.

   Arguments:

	D -		Pointer to the instance structure.
	bufPtr -	Pointer to the input buffer.
	bufLen -	Number of bytes to be written.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

	Duplicates the  input buffer into  an object and Writes  it into
	the  file. This  operation can  never  block: a  file is  always
	readable.

          Must return  a non-negative integer indicating  how many bytes
        were  written. In case  of error  must return  -1.

*/

int
ZchanOutput (ClientData D, CONST char *bufPtr, int bufLen, int *errorCodePtr)
{
  Channel *	data = (Channel *) D;
  More_Block	block;
  More_Error	error;


  if (! Ztcl_WritableFile(data->token))
    {
      *errorCodePtr = EACCES;
      return -1;
    }

  debug("request to write %d bytes to \"%s\"", bufLen,
	Tcl_GetChannelName(data->channel));

  block.ptr = (More_Byte *) bufPtr;
  block.len = bufLen;

  error = Ztcl_Write(data->token, block);
  if (! error)
    {
      debug("leaving output function, success");
      *errorCodePtr = 0;
      return block.len;
    }
  else
    {
      *errorCodePtr = More_ErrorGetInt(error);
      More_ErrorForget(error);
      return -1;
    }
}

/* ZchanSeek --

	Invoked by the  generic layer to move the  access point at which
	subsequent input or output  operations will be applied.

   Arguments:

	D -		Pointer to the instance structure.
	offset -	How far, in bytes, to move the access point.
	mode -		Selects the position for which to seek:
			SEEK_SET, SEEK_CUR or SEEK_END.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

        Returns the new access point or -1 if error.

   Side effects:

        We  know  that "SEEK_END"  is  not  supported,  but we  let  the
        interface function to handle  the error by returning the correct
        EINVAL code.

*/

int
ZchanSeek (D, offset, mode, errorCodePtr)
     ClientData		D;
     long		offset;
     int		mode;
     int *		errorCodePtr;
{
  Channel *	data = (Channel *) D;
  More_Error	error;


  debug("entering seek for channel \"%s\"",
	Tcl_GetChannelName(data->channel));

  if (offset > ((long) INT_MAX))
    {
      *errorCodePtr = EOVERFLOW;
      return -1;
    }

  error = Ztcl_Seek(data->token, ((int) offset), mode);
  if (! error)
    {
      debug("end seek, success");
      *errorCodePtr = 0;
      return offset;
    }
  else
    {
      debug("error seeking: %s", Tcl_GetString(error->errorInfo));
      *errorCodePtr = More_ErrorGetInt(error);
      More_ErrorForget(error);
      return -1;
    }
}

/* ZchanWatch --

	Invoked by the generic layer to initialise the event notification
	mechanism.

   Arguments:

	D -		Pointer to the instance data.
	mask -		An OR-ed combination of TCL_READABLE, TCL_WRITABLE
			and TCL_EXCEPTION. It indicates events the caller
			is interested in.

   Results:

        A  file  is always  readable  or  writable,  so if  interest  is
        requested  timer  handlers are  created  to  notify the  generic
        layer.

*/

void
ZchanWatch (ClientData D, int eventMask)
{
  Channel *	data = (Channel *) D;


  debug("enabling events for %s: readable %d, writable %d",
	Tcl_GetChannelName(data->channel),
	question(eventMask & TCL_READABLE),
	question(eventMask & TCL_WRITABLE));

  eventMask &= (TCL_READABLE & TCL_WRITABLE);

  if (Ztcl_ReadableFile(data->token))
    {
      if ((eventMask & TCL_READABLE) && (! data->readTimer))
	{
	  data->readTimer = \
	    Tcl_CreateTimerHandler(1, ZchanNotifyReadable, (ClientData) data);
	}
      else if ((! (eventMask & TCL_READABLE)) && (data->readTimer))
	{
	  Tcl_DeleteTimerHandler(data->readTimer);
	  data->readTimer = NULL;
	}
    }
  if (Ztcl_WritableFile(data->token))
    {
      if ((eventMask & TCL_WRITABLE) && (! data->writeTimer))
	{
	  data->writeTimer = \
	    Tcl_CreateTimerHandler(1, ZchanNotifyWritable, (ClientData) data);
	}
      else if ((! (eventMask & TCL_WRITABLE)) && (data->writeTimer))
	{
	  Tcl_DeleteTimerHandler(data->writeTimer);
	  data->writeTimer = NULL;
	}
    }
}

/* ZchanNotifyReadable --

	Invoked by the event loop  to notify the generic layer about the
	readability of the channel.

   Arguments:

	D -		File token.

   Results:

	The generic layer is notifier of the event.

*/

void
ZchanNotifyReadable (ClientData D)
{
  Channel *	data = (Channel *) D;

  Tcl_NotifyChannel(data->channel, TCL_READABLE);
  data->readTimer = NULL;
}

/* ZchanNotifyWritable --

	Invoked by the event loop  to notify the generic layer about the
	writability of the channel.

   Arguments:

	D -		File token.

   Results:

	The generic layer is notifier of the event.

*/

void
ZchanNotifyWritable (ClientData D)
{
  Channel *	data = (Channel *) D;

  Tcl_NotifyChannel(data->channel, TCL_WRITABLE);
  data->writeTimer = NULL;
}

/* ZchanGetHandle --

	Invoked by the generic layer to retrieve the device-specific
	handler for the channel.

   Arguments:

	D -		Pointer to the instance data.
	direction -	The channel direction: TCL_READABLE or
			TCL_WRITABLE.
	handlePtr -	Pointer to the variable that will hold
			the handler.

   Results:

	Returns TCL_ERROR.

*/

int
ZchanGetHandle (ClientData D, int direction, ClientData *handlePtr)
{
  return TCL_ERROR;
}

/* ZchanSetOption --

	Invoked  by the  generic layer  to set  the value  of  a channel
	specific option.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
ZchanSetOption (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;

  return More_ChannelDriverSetOption(optionTable, data->channel,
				     D, interp, optionName, optionValue);
}

/* SetOptionLevel --

	Configures the compression level of the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
SetOptionLevel (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	objPtr;
  int		level, e;

  
  objPtr = Tcl_NewStringObj(optionValue, -1);
  Tcl_IncrRefCount(objPtr);
  e = Ztcl_GetLevelFromObj(interp, objPtr, &level);
  Tcl_DecrRefCount(objPtr);
  if (TCL_OK == e)
    {
      Ztcl_SetLevel(data->token, level);
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* SetOptionStrategy --

	Configures the compression strategy of the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
SetOptionStrategy (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	objPtr;
  Ztcl_Strategy	strategy;
  int		e;

  
  objPtr = Tcl_NewStringObj(optionValue, -1);
  Tcl_IncrRefCount(objPtr);
  e = Ztcl_GetStrategyFromObj(interp, objPtr, &strategy);
  Tcl_DecrRefCount(objPtr);
  if (TCL_OK == e)
    {
      Ztcl_SetStrategy(data->token, strategy);
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* ZchanGetOption --

	Invoked by the generic layer  to retrieve the value of a channel
	specific option.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
ZchanGetOption (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  return More_ChannelDriverGetOption(optionTable, data->channel,
				     D, interp, optionName, optionValue);
}

/* GetOptionLevel --

	Extracts the compression level from the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionLevel (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  More_DStringPrintf(optionValue, "%d", Ztcl_GetLevel(data->token));
  return TCL_OK;
}

/* GetOptionStrategy --

	Extracts the compression strategy from the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionStrategy (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  More_DStringPrintf(optionValue, "%s",
		     (Ztcl_GetStrategy(data->token))->name);
  return TCL_OK;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
