/* commands.c --
   
   Part of: ZTCL
   Contents: command functions
   Date: Fri May  3, 2002
   
   Abstract
   
	In this file  are defined the callback functions  invoked by the
	TCL command [zlib].
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include <string.h>

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#ifdef ZTCL_STUB_TEST
#  include "ztclTest.h"
#else
#  include "ztclInt.h"
#endif


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug		More_Debug
#define question	More_Question

#define State(Type)	Type * state = (Type *) frame->stateStructPtr


/** ------------------------------------------------------------
 ** Command functions.
 ** ----------------------------------------------------------*/

#include "declaration.c"

#ifndef ZTCL_STUB_TEST

int
Ztcl_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;


  error = More_CreateCommands(interp, NULL, Ztcl_CommandsTable);
  return (error)? More_ErrorResult(interp, error) : TCL_OK;
}

#else
#warning using ztcl stub

int
Ztcltest_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;
  

  error = More_CreateCommands(interp, "test", Ztcl_CommandsTable);
  return (error)? More_ErrorResult(interp, error) : TCL_OK;
}

#endif


int
ZlibCompressFunction (More_CommandFrame frame)
{
  State(CompressState);
  More_Error		error;
  Tcl_Obj *		dstObj;
  int			size;


  Tcl_GetByteArrayFromObj(state->dataObj, &size);
  if (size > 0)
    {
      error = Ztcl_CompressObj(state->dataObj, &(state->config), &dstObj);
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  else
    {
      dstObj = Tcl_NewStringObj(NULL, -1);
    }

  Tcl_SetObjResult(frame->interp, dstObj);
  return TCL_OK;
}

/* ZlibDecompressFunction --

	Command function for [zlib decompress]: decompresses data stored
	in a Tcl_Obj.

   Arguments:

	frame -		Invocation frame.

   Results:

	Returns TCL_OK or TCL_ERROR. The return value of the TCL command
	is the decompressed data object.

*/

int
ZlibDecompressFunction (More_CommandFrame frame)
{
  State(DecompressState);
  More_Error		error;
  Tcl_Obj *		dstObj;
  int			size;


  Tcl_GetByteArrayFromObj(state->dataObj, &size);
  if (size > 0)
    {
      error = Ztcl_DecompressObj(state->dataObj, &(state->config),
				 state->size, &dstObj);
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  else
    {
      dstObj = Tcl_NewStringObj(NULL, -1);
    }

  Tcl_SetObjResult(frame->interp, dstObj);
  return TCL_OK;
}

/* ZlibOpenFunction --

	Opens a GZ  file for reading or writing,  not both. The synopsis
	of the TCL command is:

		zlib open <filename> <mode>

   Arguments:

	frame -		Invocation frame.

   Results:

	Returns TCL_OK or TCL_ERROR. The return value of the TCL command
	is the channel identifier.

*/

int
ZlibOpenFunction (More_CommandFrame frame)
{
  State(OpenState);
  More_Error	error;
  Ztcl_File	token;
  Tcl_Channel	channel;


  if (Tcl_IsSafe(frame->interp))
    {
      Tcl_SetResult(frame->interp, "forbidden in safe interpreter", TCL_STATIC);
      return More_RuntimeError(frame->interp);
    }

  error = Ztcl_Open(Tcl_GetString(state->fileName),
		    state->openMode, &token);
  if (error)
    {
      return More_ErrorResult(frame->interp, error);
    }
  channel = Ztcl_MakeChannel(token);

  Tcl_RegisterChannel(frame->interp, channel);
  Tcl_SetResult(frame->interp, (char *) Tcl_GetChannelName(channel), TCL_STATIC);
  return TCL_OK;
}

/* 
   Stacks a (de)compression  driver upon a channel. The  synopsis of the
   TCL command is:

		zlib stream <channel>

   The return value of the TCL command is the channel identifier.
*/

int
ZlibStreamFunction (More_CommandFrame frame)
{
  State(StreamState);
  Ztcl_Stream	input = NULL;
  Ztcl_Stream	output = NULL;
  Tcl_Channel	channel;
  More_Error	error;

  
  debug("stacking stream, readable %s, writable %s, level %d, strategy %s",
	question(state->openMode & TCL_READABLE),
	question(state->openMode & TCL_WRITABLE),
	state->config.level, state->config.strategy->name);

  if (state->openMode & TCL_READABLE)
    {
      debug("input stream compress? %s", question(state->compressInput));
      error = Ztcl_StreamInit(&input, state->compressInput, &(state->config));
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  
  if (state->openMode & TCL_WRITABLE)
    {
      debug("output stream compress? %s", question(state->compressOutput));
      error = Ztcl_StreamInit(&output, state->compressOutput, &(state->config));
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  
  channel = Ztcl_MakeTransform(input, output, state->subChannel);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewStringObj(Tcl_GetChannelName(channel), -1));
  return TCL_OK;
}

/* ZlibInfoVersionCmd --

	Accesses the ZLib library version.

   Arguments:

	frame -		Invocation frame.

   Results:

	Returns TCL_OK or TCL_ERROR; the result is a string representing
	the ZLIB version.

*/

int
ZlibInfoVersionFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, (char *)Ztcl_Version(), TCL_STATIC);
  return TCL_OK;
}

/* ZlibInfoIslevelFunction --

	Validates a compression level  selector. The synopsis of the TCL
	command is:

		zlib islevel <level>

   Arguments:

	frame -		Invocation frame.

   Results:

	Returns  TCL_OK; the  result is  true if  the value  is  a valid
	compression level selector, else returns false.

*/

int
ZlibInfoIslevelFunction (More_CommandFrame frame)
{
  State(ObjectState);
  int		e, level;

  e = Ztcl_GetLevelFromObj(frame->interp, state->objPtr, &level);
  Tcl_SetBooleanObj(Tcl_GetObjResult(frame->interp), (e == TCL_OK));
  return TCL_OK;
}

/* ZlibInfoIsstrategyFunction --

	Validates a  compression strategy selector. The  synopsis of the
	TCL command is:

		zlib isstrategy <strategy>

   Arguments:

	frame -		Invocation frame.

   Results:

	Returns  TCL_OK; the  result is  true if  the value  is  a valid
	compression strategy selector, else returns false.

*/

int
ZlibInfoIsstrategyFunction (More_CommandFrame frame)
{
  State(ObjectState);
  int		e;
  Ztcl_Strategy	strategy;

  e = Ztcl_GetStrategyFromObj(frame->interp, state->objPtr, &strategy);
  Tcl_SetBooleanObj(Tcl_GetObjResult(frame->interp), (e == TCL_OK));
  return TCL_OK;
}

/** ------------------------------------------------------------
 ** Extractors.
 ** ----------------------------------------------------------*/

static int
GetOpenModeFromArg (More_ExtractorFrame frame)
{
  return More_GetOpenModeFromObj(frame->commandFrame->interp,
				 frame->srcObj,
				 0,
				 (int *)frame->fieldPtr);
}
static int
GetLevelFromArg (More_ExtractorFrame frame)
{
  return Ztcl_GetLevelFromObj(frame->commandFrame->interp,
			      frame->srcObj,
			      (int *) frame->fieldPtr);
}
static int
GetSizeFromArg (More_ExtractorFrame frame)
{
  return More_GetUnsignedFromArg(frame);
}
static int
GetObjFromArg (More_ExtractorFrame frame)
{
  return More_GetObjFromArg(frame);
}
static int
GetChannelFromArg (More_ExtractorFrame frame)
{
  StreamState *	state = (StreamState *) frame->commandFrame->stateStructPtr;

  return More_GetChannelFromObj(frame->commandFrame->interp, frame->srcObj,
				&(state->subChannel),
				&(state->subOpenMode));
}
static int
GetStreamActionFromArg (More_ExtractorFrame frame)
{
  static CONST char *	actions[] = { "compress", "decompress", NULL };
  int			e, index;
  int *			actionPtr;


  e = Tcl_GetIndexFromObj(frame->commandFrame->interp,
			  frame->srcObj,
			  actions, "action", TCL_EXACT, &index);
  if (TCL_OK != e)
    {
      return e;
    }

  actionPtr  = (int *) frame->fieldPtr;
  *actionPtr = (! index);
  return TCL_OK;
}

/* end of file */
