/* compress.c --
   
   Part of: ztcl
   Contents: interface to ZLIB, compress functions
   Date: Sat Jan 10, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#include <limits.h>
#include "ztclInt.h"
#include "zlib.h"


/** ------------------------------------------------------------
 ** Preprocessor macros.
 ** ----------------------------------------------------------*/

#define OverflowInt(actual, delta)	\
				(((INT_MAX - (delta)) < (actual))? 1 : 0)



More_Error
Ztcl_CompressObj (srcObj, config, dstObjVar)
     Tcl_Obj *		srcObj;
     Ztcl_Config *	config;
     Tcl_Obj **		dstObjVar;
{
  int		zerror;
  More_Block	source, target;
  uLongf	srcLenLong, dstLenLong;
  Tcl_Obj *	dstObj;


  source.ptr = Tcl_GetByteArrayFromObj(srcObj, &source.len);
  if (! source.len)
    {
      *dstObjVar = Tcl_NewByteArrayObj(NULL, 0);
      return NULL;
    }

  debug("request to compress %d bytes", source.len);

  target.len = source.len - (source.len >> 1);
  if (target.len < 2048)
    {
      target.len = 4096;
    }
  debug("allocating byte array of size %d", target.len);

  dstObj = Tcl_NewByteArrayObj(NULL, 0);
  zerror = Z_OK;
  target.ptr = Tcl_SetByteArrayLength(dstObj, target.len);

  do
    {
      if (Z_BUF_ERROR == zerror)
	{
	  if (target.len < 4096)
	    {
	      target.len = 2*4096;
	    }
	  else if (INT_MAX == target.len)
	    {
	      zerror = Z_MEM_ERROR;
	      goto Error;
	    }
	  else if ((INT_MAX - target.len) <= target.len)
	    {
	      target.len = INT_MAX;
	    }
	  else
	    {
	      target.len += target.len >> 1;
	    }
	  debug("reallocating with length %d", target.len);
	}
      else if (Z_OK != zerror)
	{
	  goto Error;
	}

      target.ptr = Tcl_SetByteArrayLength(dstObj, target.len);
      srcLenLong = source.len;
      dstLenLong = target.len;
      zerror = compress2(target.ptr, &dstLenLong,
			 source.ptr, srcLenLong, config->level);
      target.len = dstLenLong;
    }
  while (Z_OK != zerror);

  debug("shrinking byte array to %d", target.len);
  Tcl_SetByteArrayLength(dstObj, target.len);
  *dstObjVar = dstObj;
  return NULL;

 Error:
  if (dstObj)
    {
      Tcl_IncrRefCount(dstObj);
      Tcl_DecrRefCount(dstObj);
    }
  return Ztcl_MakeErrorDescriptor(zerror);
}

More_Error
Ztcl_DecompressObj (srcObj, config, size, dstObjVar)
     Tcl_Obj	 *	srcObj;
     Ztcl_Config *	config;
     int		size;
     Tcl_Obj	 **	dstObjVar;
{
  int		zerror, delta;
  More_Block	source, target;
  uLongf	srcLenLong, dstLenLong;
  Tcl_Obj *	dstObj;


  source.ptr = Tcl_GetByteArrayFromObj(srcObj, &source.len);
  if (! source.len)
    {
      *dstObjVar = Tcl_NewByteArrayObj(NULL, 0);
      return NULL;
    }

  debug("request to decompress %d bytes", source.len);
  debug("suggested size %d", size);

  if (size > 0)
    {
      target.len = size;
    }
  else
    {
      delta = source.len >> 1;
      target.len = ((INT_MAX-delta) > source.len)? (source.len+delta) : INT_MAX;
      if (target.len < 2048)
	{
	  target.len = 4096;
	}
    }
  debug("allocating byte array of size %d", target.len);

  dstObj = Tcl_NewByteArrayObj(NULL, 0);
  zerror = Z_OK;

  do
    {
      if (Z_BUF_ERROR == zerror)
	{
	  if (target.len < 4096)
	    {
	      target.len = 2*4096;
	    }
	  else if (INT_MAX == target.len)
	    {
	      zerror = Z_MEM_ERROR;
	      goto Error;
	    }
	  else if ((INT_MAX - target.len) < target.len)
	    {
	      target.len = INT_MAX;
	    }
	  else
	    {
	      target.len <<= 1;
	    }
	  debug("reallocating byte array of size %d", target.len);
	}
      else if (Z_OK != zerror)
	{
	  goto Error;
	}

      target.ptr = Tcl_SetByteArrayLength(dstObj, target.len);

      srcLenLong = source.len;
      dstLenLong = target.len;
      zerror = uncompress(target.ptr, &dstLenLong,
			  source.ptr, srcLenLong);
      target.len = dstLenLong;
    }
  while (Z_OK != zerror);

  debug("shrinking byte array to %d", target.len);
  Tcl_SetByteArrayLength(dstObj, target.len);
  *dstObjVar = dstObj;
  return NULL;

 Error:
  if (dstObj)
    {
      Tcl_IncrRefCount(dstObj);
      Tcl_DecrRefCount(dstObj);
    }
  return Ztcl_MakeErrorDescriptor(zerror);
}

/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
