/*
  This file is meant to be included (with #include) in a source
  file provided by you that implements the command functions. Your
  file must include the TCL header ("tcl.h") first and define the
  functions later.

    The reason of this inclusion is that here are declared the struct
  typedefs of the command invocation states. These types must be
  visible by the command functions. The stuff declared in this file
  uses the "static" keyword as much as possible.

    The command functions should be small: they should do the job by
  invoking other functions, the ones that the extension should export
  in its stub table.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "tclmore.h"



/** ------------------------------------------------------------
 ** Type declarations.
 ** ----------------------------------------------------------*/

/* CompressState --

  This structure is used to hold the invocation state of the
  following commands:

                    [zlib compress   data ?options?]
*/
typedef struct CompressState {
  Tcl_Obj *               dataObj;
  Ztcl_Config             config;
} CompressState;

static CONST CompressState CompressStateDefaults = {
  NULL, ZTCL_DEFAULT_CONFIG
};

/* DecompressState --

  This structure is used to hold the invocation state of the
  following commands:

                    [zlib decompress   data ?options?]
*/
typedef struct DecompressState {
  Tcl_Obj *               dataObj;
  int                     size;
  Ztcl_Config             config;
} DecompressState;

static CONST DecompressState DecompressStateDefaults = {
  NULL, 0, ZTCL_DEFAULT_CONFIG
};

/* OpenState --

  This structure is used to hold the invocation state of the
  following commands:

                    [zlib open   file mode]
*/
typedef struct OpenState {
  Tcl_Obj *               fileName;
  int                     openMode;
  Ztcl_Config             config;
} OpenState;

static CONST OpenState OpenStateDefaults = {
  NULL, 0, ZTCL_DEFAULT_CONFIG
};

/* ObjectState --

  This structure is used to hold the invocation state of the
  following commands:

                    [zlib info islevel]
                    [zlib info isstrategy]
*/
typedef struct ObjectState {
  Tcl_Obj *               objPtr;
} ObjectState;

static CONST ObjectState ObjectStateDefaults = {
  NULL
};

/* StreamState --

  This structure is used to hold the invocation state of the
  following commands:

                    [zlib stream   channel mode ?options?]
*/
typedef struct StreamState {
  Tcl_Channel             subChannel;
  int                     subOpenMode;
  int                     openMode;
  int                     compressInput;
  int                     compressOutput;
  Ztcl_Config             config;
} StreamState;

static CONST StreamState StreamStateDefaults = {
  NULL, 0, 0, 1, 0, ZTCL_DEFAULT_CONFIG
};



/** ------------------------------------------------------------
 ** Implementation functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of these functions in
  this file. When the function is invoked, the state invocation
  structure has been already filled with values from the command line
  arguments and options, with no errors.

    The function should look like the following:

  int
  MyCommandFunction (More_CommandFrame frame)
  {
     MyStateStruct *  state = (MyStateStruct *) frame->stateStructPtr;
     Tcl_Obj *        result;

     *
       Make use of the fields of "*state" to build a result by
       invoking functions from your library.
     *
     result = Mylib_DoSomething(state->value, state->another_value);

     Tcl_SetObjResult(frame->interp, result);
     return TCL_OK;
  }

    Being the implementation of a TCL command, this function is
  declared "static": it is useless to share this function among
  different modules. Remember that the role of a "More_CmdFunc"
  is to be an interface between the state invocation structure and
  the functions of your library that do the "real work".
*/

static More_CmdFunc         ZlibCompressFunction;
static More_CmdFunc         ZlibDecompressFunction;
static More_CmdFunc         ZlibOpenFunction;
static More_CmdFunc         ZlibStreamFunction;
static More_CmdFunc          ZlibInfoIslevelFunction;
static More_CmdFunc          ZlibInfoIsstrategyFunction;
static More_CmdFunc          ZlibInfoVersionFunction;


/** ------------------------------------------------------------
 ** Extractor functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of the static extractors
  in this file. Functions declared EXTERN can be implemented in other
  modules of your extension or in other libraries that are dependencies
  of this one (for example the TCLMORE library implements some common
  extractors).

    The functions must be available as pointers: you cannot directly use
  functions exported in a stub table; this is because the pointer must
  be included in a statically allocated data structure. Remember that
  stub function names are available as macros, each macro references
  an element in a vector of function pointers in another library: the
  value of the function pointer is available only at execution time.

    The only way to use an extractor from a stub table is to declare
  another extractor (with a similar name) that wraps the real function.
  Example:

     More_Extractor MyExtractor;
     ...
     int
     MyExtractor (More_ExtractorFrame frame)
     {
       return RealExtractor(frame);
     }

  "RealExtractor" can be the macro of a stub function.

    If a prototype is omitted: it is your responsibility to declare it
  before the line that includes this file in your source module. Obviously
  the declaration can be in an external header file; this is particularly
  useful if your extension is using a wrapper extractor for a real extractor
  from another extension, and the extractor is used in more than one
  set of commands.

    A common extractor has the following structure, lets assume that
  "Type" is the type of value we want:

     EXTERN int Mylib_GetTypeFromObj _ANSI_ARGS_((Tcl_Interp * interp,
                                                  Tcl_Obj *    srcObj,
                                                  Type *       valueVar));
     ...
     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        return Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                    frame->srcObj,
                                    (Type *) frame->fieldPtr);       
     }

  or:

     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        int   e;

        e = Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                 frame->srcObj,
                                 (Type *) frame->fieldPtr);       
        return ((TCL_OK == e)? e : More_LogicError(interp));
     }

  the implementation of a "GetTypeFromObj" function to do the real
  work is recommended because such functions can be used by other
  modules, and even exported in the stub table.

    Remember that the field pointer can reference a whole structure
  embedded in the state invocation one.

    At present an extractor can validate arguments only with complete
  independence from the current process state, that is it can test that
  a value is in the correct space and range. Coherence tests between
  argument values and process state must be performed by the command
  implementation functions. This is not exactly true, but it is the
  recommended way to do things.
*/

static More_Extractor      GetObjFromArg;
static More_Extractor      GetLevelFromArg;
static More_Extractor      GetSizeFromArg;
static More_Extractor      GetOpenModeFromArg;
static More_Extractor      GetChannelFromArg;
static More_Extractor      GetStreamActionFromArg;


/** ------------------------------------------------------------
 ** Command arguments tables.
 ** ----------------------------------------------------------*/

#define zlib_compress_ARGUMENT_NUMBER	1
static CONST More_CommandArgument ZlibCompressArguments[1] = {
  { GetObjFromArg,       offsetof(struct CompressState, dataObj) }
};

#define zlib_decompress_ARGUMENT_NUMBER	1
static CONST More_CommandArgument ZlibDecompressArguments[1] = {
  { GetObjFromArg,       offsetof(struct DecompressState, dataObj) }
};

#define zlib_open_ARGUMENT_NUMBER	2
static CONST More_CommandArgument ZlibOpenArguments[2] = {
  { GetObjFromArg,       offsetof(struct OpenState, fileName) },
  { GetOpenModeFromArg,  offsetof(struct OpenState, openMode) }
};

#define zlib_stream_ARGUMENT_NUMBER	2
static CONST More_CommandArgument ZlibStreamArguments[2] = {
  { GetChannelFromArg,   offsetof(struct StreamState, subChannel) },
  { GetOpenModeFromArg,  offsetof(struct StreamState, openMode) }
};

#define zlib_info_islevel_ARGUMENT_NUMBER	1
static CONST More_CommandArgument ZlibInfoIslevelArguments[1] = {
  { GetObjFromArg,       offsetof(struct ObjectState, objPtr) }
};

#define zlib_info_isstrategy_ARGUMENT_NUMBER	1
static CONST More_CommandArgument ZlibInfoIsstrategyArguments[1] = {
  { GetObjFromArg,       offsetof(struct ObjectState, objPtr) }
};

/* No arguments for [zlib info version]. */
#define zlib_info_version_ARGUMENT_NUMBER	0



/** ------------------------------------------------------------
 ** Command options tables.
 ** ----------------------------------------------------------*/

static CONST More_CommandOption ZlibCompressOptions[2] = {
  { "-level", 1, GetLevelFromArg, offsetof(struct CompressState, config.level) },
  { NULL, 0, NULL, 0 }
};

static CONST More_CommandOption ZlibDecompressOptions[2] = {
  { "-size", 1, GetSizeFromArg, offsetof(struct DecompressState, size) },
  { NULL, 0, NULL, 0 }
};

/* No options for [zlib open]. */

static CONST More_CommandOption ZlibStreamOptions[4] = {
  { "-input", 1, GetStreamActionFromArg, offsetof(struct StreamState, compressInput) },
  { "-output", 1, GetStreamActionFromArg, offsetof(struct StreamState, compressOutput) },
  { "-level", 1, GetLevelFromArg, offsetof(struct StreamState, config.level) },
  { NULL, 0, NULL, 0 }
};

/* No options for [zlib info islevel]. */

/* No options for [zlib info isstrategy]. */

/* No options for [zlib info version]. */


/** ------------------------------------------------------------
 ** Leaf commands array.
 ** ----------------------------------------------------------*/

/*
  There is a leaf structure for each "More_CmdFunc". Each structure
  has references to the argument and option tables. It is not your
  business to mess with this structure.
*/

static More_CommandTreeLeaf commandTreeLeaves[] = {
  /* (0) synopsis: zlib compress data ?options? */
  { 2, ZlibCompressFunction, sizeof(CompressState), (CONST ClientData)&CompressStateDefaults,
    zlib_compress_ARGUMENT_NUMBER, ZlibCompressArguments, ZlibCompressOptions,
    "data ?options?" },

  /* (1) synopsis: zlib decompress data ?options? */
  { 2, ZlibDecompressFunction, sizeof(DecompressState), (CONST ClientData)&DecompressStateDefaults,
    zlib_decompress_ARGUMENT_NUMBER, ZlibDecompressArguments, ZlibDecompressOptions,
    "data ?options?" },

  /* (2) synopsis: zlib open file mode */
  { 2, ZlibOpenFunction, sizeof(OpenState), (CONST ClientData)&OpenStateDefaults,
    zlib_open_ARGUMENT_NUMBER, ZlibOpenArguments, NULL,
    "file mode" },

  /* (3) synopsis: zlib stream channel mode ?options? */
  { 2, ZlibStreamFunction, sizeof(StreamState), (CONST ClientData)&StreamStateDefaults,
    zlib_stream_ARGUMENT_NUMBER, ZlibStreamArguments, ZlibStreamOptions,
    "channel mode ?options?" },

  /* (4) synopsis: zlib info islevel  */
  { 3, ZlibInfoIslevelFunction, sizeof(ObjectState), (CONST ClientData)&ObjectStateDefaults,
    zlib_info_islevel_ARGUMENT_NUMBER, ZlibInfoIslevelArguments, NULL,
    NULL },

  /* (5) synopsis: zlib info isstrategy  */
  { 3, ZlibInfoIsstrategyFunction, sizeof(ObjectState), (CONST ClientData)&ObjectStateDefaults,
    zlib_info_isstrategy_ARGUMENT_NUMBER, ZlibInfoIsstrategyArguments, NULL,
    NULL },

  /* (6) synopsis: zlib info version  */
  { 3, ZlibInfoVersionFunction, 0, (CONST ClientData)NULL,
    zlib_info_version_ARGUMENT_NUMBER, NULL, NULL,
    NULL }
};


/** ------------------------------------------------------------
 ** Node commands array.
 ** ----------------------------------------------------------*/

/*
  These are node structures that declare subcommands with subcommands.
  It is not your business to mess with this structure.
*/

static More_CommandTreeNode ZlibInfoCommandNode[] = {
  { "islevel", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[4] },
  { "isstrategy", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[5] },
  { "version", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[6] },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};

static More_CommandTreeNode ZlibCommandNode[] = {
  { "compress", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[0] },
  { "decompress", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[1] },
  { "open", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[2] },
  { "stream", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[3] },
  { "info", TCLMORE_SAFE_COMMAND,  ZlibInfoCommandNode, NULL },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};



/** ------------------------------------------------------------
 ** Root commands table.
 ** ----------------------------------------------------------*/

/*
  This array of structures declares the root commands. There is one of
  them for each <maincommand> and <command> markups with no ancestors
  in the original XML document. It is not your business to mess with
  the content of this array.

    You may create commands declared in this array with the following
  chunk of code:

    error = More_CreateCommands(interp, "namesp", Ztcl_CommandsTable);
    if (error)
      {
	return More_ErrorResult(interp, error);
      }

  where "namesp" is the name of the namespace in which the command
  must be created. If the namespace is the root one: you can use NULL.
*/

More_CommandTreeNode Ztcl_CommandsTable[] = {
  { "zlib", TCLMORE_SAFE_COMMAND, ZlibCommandNode, NULL },
  { NULL, 0, NULL, NULL }
};


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/

