/* gzfile.c --
   
   Part of: ZTCL
   Contents: interface to ZLIB, gzfile functions
   Date: Tue Apr 30, 2002
   
   Abstract

   	This file defines  the interface to the functions  of ZLIB. This
   	is one of the files that have access to the "zlib.h" header.

   Copyright (c)  2002, 2003, 2004  Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#include <limits.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include "ztclInt.h"
#include "zlib.h"


/** ------------------------------------------------------------
 ** Macros.
 ** ----------------------------------------------------------*/

#define DESCRIPTOR(file)	((gzFile)(file)->descriptor)

#define READING_MODE	0
#define WRITING_MODE	1

#ifndef Ztcl_Offset
#  define Ztcl_Offset		z_off_t
#endif


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct File {
  gzFile	handle;
  int		openMode;
  Ztcl_Config	config;
} File;


/** ------------------------------------------------------------
 ** Module's function prototypes.
 ** ----------------------------------------------------------*/

static More_Error	GzError		_ANSI_ARGS_((File *file));


More_Error
Ztcl_Open (CONST char *fileName, int openMode, Ztcl_File *tokenVar)
{
  gzFile	handle;
  File *	file;
  

  openMode &= (TCL_READABLE | TCL_WRITABLE);
  assert(openMode);

  handle = gzopen(fileName, ((openMode & TCL_READABLE)? "rb" : "wb"));
  if (! handle)
    {
      if (Tcl_GetErrno())
	{
	  return More_ErrorErrno();
	}
      else
	{
	  More_Error	error = More_ErrorNoMemory();
	  More_ErrorSetInt(error, ENOMEM);
	  return error;
	}
    }

  file = (File *) ckalloc(sizeof(struct File));
  memset(file, '\0', sizeof(struct File));
  file->handle		= handle;
  file->openMode	= openMode;
  file->config.level	= ZTCL_LEVEL_DEFAULT;
  file->config.strategy	= ZTCL_STRATEGY_DEFAULT;

  if (openMode & TCL_WRITABLE)
    {
      gzsetparams(handle, ZTCL_LEVEL_DEFAULT, Z_DEFAULT_STRATEGY);
    }
  debug("handle %p", file->handle);
  *tokenVar = (Ztcl_File) file;
  return NULL;
}

More_Error
Ztcl_Close (Ztcl_File token)
{
  File *	file  = (File *) token;
  More_Error	error = NULL;


  debug("handle %p", file->handle);
  if (Z_OK != gzclose(file->handle))
    {
      ckfree((char *) file);
      error = GzError(file);
    }
  return error;
}

More_Error
Ztcl_Read (Ztcl_File token, More_Block *blockVar)
{
  File *	file = (File *) token;
  More_Block	block;


  block = *blockVar;
  if (block.len <= 0)
    {
      blockVar->len = 0;
      return NULL;
    }

  block.len = gzread(file->handle, block.ptr, (unsigned) block.len);
  if (-1 == block.len)
    {
      blockVar->len = 0;
      return GzError(file);
    }

  *blockVar = block;
  return NULL;
}

More_Error
Ztcl_ReadObj (Ztcl_File token, int number, Tcl_Obj **dstObjVar)
{
  More_Block	block;
  Tcl_Obj *	dstObj;
  More_Error	error;


  dstObj = Tcl_NewByteArrayObj(NULL, 0);
  block.len = number;
  block.ptr = Tcl_SetByteArrayLength(dstObj, block.len);
  error = Ztcl_Read(token, &block);
  if (error) { goto Error_Reading; }
  Tcl_SetByteArrayLength(dstObj, block.len);
  *dstObjVar = dstObj;
  return NULL;

 Error_Reading:
  Tcl_IncrRefCount(dstObj);
  Tcl_DecrRefCount(dstObj);
  return GzError((File *) token);
}

More_Error
Ztcl_Write (Ztcl_File token, More_Block block)
{
  File *	file = (File *) token;
  

  if (0 >= block.len) { return NULL; }
  return ((gzwrite(file->handle, block.ptr, (unsigned) block.len))? \
	  NULL : GzError(file));
}

More_Error
Ztcl_WriteObj (Ztcl_File token, Tcl_Obj *srcObj)
{
  More_Block	block;

  block.ptr = Tcl_GetByteArrayFromObj(srcObj, &block.len);
  return Ztcl_Write(token, block);
}

int
Ztcl_Eof (Ztcl_File token)
{
  File * file = (File *) token;
  return gzeof(file->handle);
}

More_Error
Ztcl_Flush (Ztcl_File token)
{
  File * file = (File *) token;

  return ((Z_OK == gzflush(file->handle, Z_FULL_FLUSH))? NULL : GzError(file));
}

More_Error
Ztcl_Seek (Ztcl_File token, int offset, int whence)
{
  File * file = (File *) token;
  More_Error	error;


  switch (whence)
    {
    case SEEK_SET:
    case SEEK_CUR:
      break;

    default: /* SEEK_END */
      goto Error;
    }
  if (-1 == gzseek(file->handle, (z_off_t)offset, whence))
    {
      goto Error;
    }
  return NULL;

 Error:
  error = More_ErrorNew();
  More_ErrorLogicStr(error, "invalid seek operation");
  More_ErrorSetInt(error, EINVAL);
  return error;
}

More_Error
Ztcl_Tell (Ztcl_File token, int *positionVar)
{
  File * file = (File *) token;
  More_Error	error;
  Ztcl_Offset	position;


  position = ((Ztcl_Offset) gztell(file->handle));
  if (((Ztcl_Offset) -1) != position)
    {
      if (position > ((Ztcl_Offset) INT_MAX))
	{
	  goto Error_Overflow;
	}

      *positionVar = (int) position;
      return NULL;
    }

  return GzError(file);

 Error_Overflow:
  error = More_ErrorNew();
  More_ErrorSetInt(error, EOVERFLOW);
  More_ErrorRuntimeStr(error, "offset too big");
  return error;
}

More_Error
Ztcl_SetLevel (Ztcl_File token, int level)
{
  File * file	= (File *) token;
  int		e;
  More_Error	error;


  e = gzsetparams(file->handle, level, file->config.strategy->value);
  if (Z_OK != e)
    {
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "invalid value");
      More_ErrorSetInt(error, EINVAL);
      return error;
    }

  file->config.level = level;
  return NULL;
}

int
Ztcl_GetLevel (Ztcl_File token)
{
  File * file	= (File *) token;
  return file->config.level;
}
More_Error
Ztcl_SetStrategy (Ztcl_File token, Ztcl_Strategy strategy)
{
  File *	file = (File *) token;
  int		e;
  More_Error	error;


  if (! strategy) { strategy = ZTCL_STRATEGY_DEFAULT; }
  e = gzsetparams(file->handle, file->config.level, strategy->value);
  if (Z_OK != e)
    {
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "invalid value");
      More_ErrorSetInt(error, EINVAL);
      return error;
    }

  file->config.strategy = strategy;
  return NULL;
}

Ztcl_Strategy
Ztcl_GetStrategy (Ztcl_File token)
{
  File *	file = (File *) token;
  return file->config.strategy;
}

int
Ztcl_ReadableFile (Ztcl_File token)
{
  return (TCL_READABLE == ((File *) token)->openMode);
}
int
Ztcl_WritableFile (Ztcl_File token)
{
  return (TCL_WRITABLE == ((File *) token)->openMode);
}

More_Error
GzError (File *file)
{
  CONST char *	errorString;
  int		errorCode;
  More_Error	error;


  errorString = gzerror(file->handle, &errorCode);
  error = More_ErrorNew();
  More_ErrorRuntimeStr(error, ((Z_ERRNO == errorCode)?
			       Tcl_ErrnoMsg(Tcl_GetErrno()) : errorString));
  More_ErrorSetInt(error, ((Z_ERRNO == errorCode)? Tcl_GetErrno() : EIO));
  gzclearerr(file->handle);
  return error;
}

/* end of file */
/*
  Local Variables:
  page-delimiter: "^"
  End:
*/
