/*  miscellaneous.c --
   
   Part of: ztcl
   Contents: miscellaneous functions
   Date: Wed Mar 10, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "ztclInt.h"
#include <errno.h>
#include <assert.h>
#include <zlib.h>


/** ------------------------------------------------------------
 ** Module variables.
 ** ----------------------------------------------------------*/

#define NUMBER_OF_STRATEGIES		4
CONST struct Ztcl_StrategyTag Ztcl_Strategies[NUMBER_OF_STRATEGIES] = {
  { "default",		Z_DEFAULT_STRATEGY },
  { "filtered",		Z_FILTERED },
  { "huffman",		Z_HUFFMAN_ONLY },
  { "rle",		Z_RLE },
  { NULL, 0 }
};


int
Ztcl_GetLevelFromObj (interp, objPtr, levelPtr)
     Tcl_Interp *	interp;
     Tcl_Obj	*	objPtr;
     int	*	levelPtr;
{
  int		e, level;
  CONST char *	str;
  

  str = Tcl_GetString(objPtr);
  e = Tcl_GetIntFromObj(interp, objPtr, &level);
  if (e == TCL_OK)
    {
    }
  else if (strcmp("default", str) == 0)
    {
      level = ZTCL_LEVEL_DEFAULT;
    }
  else if (strcmp("fast", str) == 0)
    {
      level = ZTCL_LEVEL_FAST;
    }
  else if (strcmp("best", str) == 0)
    {
      level = ZTCL_LEVEL_BEST;
    }
  else
    {
      goto Error;
    }

  if (Ztcl_IsLevel(level))
    {
      *levelPtr = level;
      return TCL_OK;
    }

 Error:
  if (interp != NULL)
    {
      Tcl_ResetResult(interp);
      Tcl_AppendResult(interp, "wrong compression level \"",
		       Tcl_GetString(objPtr), "\"", NULL);
    }
  return TCL_ERROR;
}

int
Ztcl_IsLevel (int level)
{
  return ((0 <= level) && (level <= 9));
}

int
Ztcl_GetStrategyFromObj (interp, objPtr, strategyPtr)
     Tcl_Interp *	interp;
     Tcl_Obj *		objPtr;
     Ztcl_Strategy *	strategyPtr;
{
  int			e, index;


  e = Tcl_GetIndexFromObjStruct(interp, objPtr, Ztcl_Strategies,
				sizeof(struct Ztcl_StrategyTag),
				"strategy", 0, &index);
  if (e != TCL_OK)
    {
      return TCL_ERROR;
    }
	  
  *strategyPtr = &(Ztcl_Strategies[index]);
  return TCL_OK;
}

int
Ztcl_IsStrategy (Ztcl_Strategy strategy)
{
  int	i;

  for (i=0; i<(NUMBER_OF_STRATEGIES-1); ++i)
    {
      if (&(Ztcl_Strategies[i]) == strategy)
	{
	  return 1;
	}
    }
  return 0;
}

/* Ztcl_Version --

	Accesses the version number.

   Arguments:

	None.

   Results:

        Returns a pointer to  a statically allocated string representing
        the ZLib version number.

   Side effects:

        None.

*/

CONST char *
Ztcl_Version (void)
{
  return zlibVersion();
}

More_Error
Ztcl_MakeErrorDescriptor (int zerror)
{
  More_Error	error = More_ErrorNew();
  CONST char *	errorString;
  int		errorCode;


  debug("building error descriptor");
  assert(Z_OK != zerror);
  switch (zerror)
    {
    case Z_ERRNO:
      debug("Z_ERRNO");
      errorCode		= Tcl_GetErrno();
      errorString	= Tcl_ErrnoMsg(errorCode);
      goto Error_Runtime;

    case Z_STREAM_ERROR:
      debug("Z_STREAM_ERROR");
      errorString	= "stream error";
      errorCode		= EINVAL;
      goto Error_Logic;

    case Z_MEM_ERROR:
      debug("Z_MEM_ERROR");
      errorString	= "not enough memory";
      errorCode		= ENOMEM;
      goto Error_Runtime;
      
    case Z_DATA_ERROR:
      debug("Z_DATA_ERROR");
      errorString	= "corrupted input data";
      errorCode		= EINVAL;
      goto Error_Runtime;

    case Z_BUF_ERROR:
      debug("Z_BUF_ERROR");
      errorString	= "input or output buffer error";
      errorCode		= EINVAL;
      goto Error_Runtime;

    case Z_VERSION_ERROR:
      debug("Z_VERSION_ERROR");
      errorString	= "incompatible library version";
      errorCode		= EINVAL;
      goto Error_Runtime;

    default:
      debug("unknown internal error (%d)", zerror);
      errorString	= "unknown internal error";
      errorCode		= EINVAL;
      goto Error_Logic;
    }
  
 Error_Logic:
  More_ErrorLogicStr(error, errorString);
  goto Error;
 Error_Runtime:
  More_ErrorRuntimeStr(error, errorString);
 Error:
  More_ErrorSetInt(error, errorCode);
  return error;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
