/*  stream.c --
   
   Part of: ztcl
   Contents: stream functions
   Date: Wed Mar 10, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#include <assert.h>
#include <errno.h>
#include <stdarg.h>
#include "ztclInt.h"
#include "zlib.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question		More_Question

#define OUTPUT_BUFFER_SIZE		(4096*4)
#define OUTPUT_BUFFER_HYSTERESIS	(4096*4)


#define OutputBufferGetNumberOfUsedBytes(stream)		\
	((stream)->output.allocated.len - (stream)->zstream.context.avail_out)
#define OutputBufferIsAllocated(stream)	(NULL != (stream)->output.allocated.ptr)
#define OutputBufferFree(stream)	ckfree((char *)(stream)->output.allocated.ptr)

#define MakeErrorDescriptor	Ztcl_MakeErrorDescriptor


/* z context macros */

#define ContextInputDataIsAvailable(context)	(context->avail_in > 0)
#define ContextUnusedLength(context)		(context->avail_out)


/** ------------------------------------------------------------
 ** Debug message macros.
 ** ----------------------------------------------------------*/

#define Debug_PrintStreamType(stream)		\
	debug("%s stream (stream=%p)",	\
		((stream)->driver == &CompressionDriver)? "compress" : "decompress", \
		(stream))

#define Debug_PrintAvailableData(stream, prefix)	\
	debug("%s: avail_in %d, output buffer: size=%d, used=%d, avail_out=%d",	\
		(prefix),					\
		(stream)->zstream.context.avail_in,		\
		(stream)->output.allocated.len,			\
		OutputBufferGetNumberOfUsedBytes(stream),	\
		(stream)->zstream.context.avail_out)



/** ------------------------------------------------------------
 ** Assertion macros.
 ** ----------------------------------------------------------*/

#define Assert_ValidateOutputBuffer(stream)		\
	assert((0 == (stream)->zstream.context.avail_out) || \
               (((stream)->output.allocated.ptr + (stream)->output.allocated.len) > \
	        (More_BytePtr)(stream)->zstream.context.next_out));		\
	assert((More_BytePtr)((stream)->zstream.context.next_out +		\
		(stream)->zstream.context.avail_out) == \
               ((stream)->output.allocated.ptr + (stream)->output.allocated.len))

#define Assert_OffsetInOutputBuffer(stream, offset)	\
	assert((stream)->output.allocated.len - \
		(stream)->zstream.context.avail_out >= (offset))


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Stream		Stream;
typedef struct ZStream		ZStream;


typedef int DriverInit		_ANSI_ARGS_((ZStream *bzstream));
typedef int DriverWrite		_ANSI_ARGS_((Stream *stream));
typedef int DriverFlush		_ANSI_ARGS_((Stream *stream));
typedef int DriverFinish	_ANSI_ARGS_((Stream *stream));

typedef struct Driver {
  DriverInit *			init;
  DriverWrite *			write;
  DriverFlush *			flush;
  DriverFinish *		finish;
} Driver;


struct ZStream {
  z_stream		context;
  Ztcl_Config		config;
};

typedef struct OutputBuffer {
  More_Block		allocated;
  int			size;
  int			hysteresis;
} OutputBuffer;

struct Stream {
  ZStream		zstream;
  Driver *		driver;
  OutputBuffer		output;
  int			finished;
};


/** ------------------------------------------------------------
 ** Module's function prototypes.
 ** ----------------------------------------------------------*/

static DriverInit	CompressInit;
static DriverWrite	CompressWrite;
static DriverFlush	CompressFlush;
static DriverFinish	CompressFinish;

static DriverInit	DecompressInit;
static DriverWrite	DecompressWrite;
static DriverFlush	DecompressFlush;
static DriverFinish	DecompressFinish;

static int DecompressCheckEndOfStream	_ANSI_ARGS_((Stream *stream,
						     int zerror));

static void OutputBufferEnlarge	_ANSI_ARGS_((Stream *stream));
static void OutputBufferShrink	_ANSI_ARGS_((Stream *stream));


static void * StreamAlloc	_ANSI_ARGS_((void *opaque, uInt number, uInt size));
static void   StreamFree	_ANSI_ARGS_((void *opaque, void *block));

static More_Error RuntimeError	_ANSI_ARGS_((CONST char *errorInfo,
					     int errorCode));



/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Driver CompressionDriver = {
  CompressInit,
  CompressWrite,
  CompressFlush,
  CompressFinish
};

static Driver DecompressionDriver = {
  DecompressInit,
  DecompressWrite,
  DecompressFlush,
  DecompressFinish
};


More_Error
Ztcl_StreamInit (Ztcl_Stream *tokenVar, int compress, Ztcl_Config *config)
{
  Stream *	stream;
  More_Error	error = NULL;
  int		zerror;


  debug("creating stream, compression? %s", question(compress));

  More_AllocStruct(stream, Stream);
  stream->driver = (compress)? &CompressionDriver : &DecompressionDriver;
  stream->output.size			= OUTPUT_BUFFER_SIZE;
  stream->output.hysteresis		= OUTPUT_BUFFER_HYSTERESIS;
  stream->zstream.context.zalloc	= StreamAlloc;
  stream->zstream.context.zfree		= StreamFree;
  stream->zstream.config		= *config;

  zerror = stream->driver->init(&(stream->zstream));
  if (Z_OK == zerror)
    {
      *tokenVar = (Ztcl_Stream) stream;
    }
  else
    {
      ckfree((char *) stream);
      error = MakeErrorDescriptor(zerror);
    }
  return error;
}
static int
CompressInit (ZStream *zstream)
{
  return deflateInit(&(zstream->context), zstream->config.level);
}
static int
DecompressInit (ZStream *zstream)
{
  return inflateInit(&(zstream->context));
}

void
Ztcl_StreamFinal (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;


  debug("finalising stream, to finish? %s", question(! stream->finished));
  if (! stream->finished)
    {
      stream->driver->finish(stream);
    }
  if (OutputBufferIsAllocated(stream))
    {
      debug("freeing buffer");
      OutputBufferFree(stream);
    }
  ckfree((char *) stream);
}

More_Block
Ztcl_StreamOutput (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  More_Block	block;


  block.len = OutputBufferGetNumberOfUsedBytes(stream);
  block.ptr = stream->output.allocated.ptr;
  return block;
}

void
Ztcl_StreamRead (Ztcl_Stream token, int numberOfReadBytes)
{
  Stream *	stream		= (Stream *) token;
  More_Block *	outputBuffer	= &(stream->output.allocated);
  z_stream *	context		= &(stream->zstream.context);
  size_t	numberOfUnreadBytes;


  Assert_ValidateOutputBuffer(stream);
  Assert_OffsetInOutputBuffer(stream, numberOfReadBytes);
  debug("commit read: %d bytes were read", numberOfReadBytes);

  /*
                              outputBuffer->len
    |-------------------------------------------------------------------|
    <...................><___________________><.........................>
      numberOfReadBytes   numberOfUnreadBytes      context->avail_out
  */

  numberOfUnreadBytes = \
    outputBuffer->len - numberOfReadBytes - context->avail_out;

  memmove(outputBuffer->ptr, outputBuffer->ptr + numberOfReadBytes,
	  numberOfUnreadBytes);

  context->next_out  -= numberOfReadBytes;
  context->avail_out += numberOfReadBytes;

  Assert_ValidateOutputBuffer(stream);
  Debug_PrintAvailableData(stream, "after reading data");
  OutputBufferShrink(stream);
}

More_Error
Ztcl_StreamWrite (Ztcl_Stream token, More_Block *inputPtr)
{
  Stream *	stream = (Stream *) token;
  z_stream *	context = &(stream->zstream.context);
  int		zerror;
  More_Error	error  = NULL;


  if ((inputPtr->len > 0) && (! stream->finished))
    {
      Debug_PrintStreamType(stream);
      debug("request to write %d bytes", inputPtr->len);
      context->next_in  = inputPtr->ptr;
      context->avail_in = inputPtr->len;
      zerror = stream->driver->write(stream);
      if (Z_OK == zerror)
	{
	  if (ContextInputDataIsAvailable(context))
	    {
	      inputPtr->len = context->avail_in;
	      inputPtr->ptr = context->next_in;
	      debug("unread input %d bytes", inputPtr->len);
	    }
	  else
	    {
	      More_BlockReset(*inputPtr);
	    }
	  debug("end operation");
	}
      else
	{
	  debug("write error: %d", zerror);
	  error = MakeErrorDescriptor(zerror);
	}
      context->next_in  = NULL;
      context->avail_in = 0;
    }
  Debug_PrintAvailableData(stream, "after writing");
  return error;
}

static int
CompressWrite (Stream *stream)
{
  int		zerror;
  z_stream *	context = &(stream->zstream.context);


  debug("compress write loop");
  do
    {
      OutputBufferEnlarge(stream);
      zerror = deflate(context, 0);
    }
  while (ContextInputDataIsAvailable(context) && (Z_OK == zerror));
  return zerror;
}
static int
DecompressWrite (Stream *stream)
{
  int		zerror;
  z_stream *	context = &(stream->zstream.context);


  debug("decompress write loop");
  do
    {
      OutputBufferEnlarge(stream);
      zerror = inflate(context, 0);
    }
  while (ContextInputDataIsAvailable(context) && (Z_OK == zerror));
  return DecompressCheckEndOfStream(stream, zerror);
}

More_Error
Ztcl_StreamFinish (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  int		zerror;


  if (! stream->finished)
    {
      stream->finished = 1;
      zerror = stream->driver->finish(stream);
      debug("after finishing: code %d", zerror);
      if (Z_STREAM_END != zerror)
	{
	  return (Z_OK == zerror)? \
	    RuntimeError("end of stream not found", EINVAL) : \
	    MakeErrorDescriptor(zerror);
	}
      Debug_PrintAvailableData(stream, "after finishing");
      Assert_ValidateOutputBuffer(stream);
    }
  return NULL;
}

static int
CompressFinish (Stream *stream)
{
  int		zerror;
  z_stream *	context = &(stream->zstream.context);


  debug("finishing compress stream");
  do
    {
      OutputBufferEnlarge(stream);
      zerror = deflate(context, Z_FINISH);
    }
  while (Z_OK == zerror);
  deflateEnd(context);
  return zerror;
}
static int
DecompressFinish (Stream *stream)
{
  int		zerror, outputAvailableBefore, outputAvailableAfter;
  z_stream *	context = &(stream->zstream.context);


  debug("finishing decompress stream");
  do
    {
      OutputBufferEnlarge(stream);
      outputAvailableBefore = context->avail_out;
      zerror = inflate(context, Z_FINISH);
      outputAvailableAfter  = context->avail_out;
      debug("input data: before %d, after %d",
	    outputAvailableBefore, outputAvailableAfter);
    }
  while ((Z_OK == zerror) && (outputAvailableBefore < outputAvailableAfter));
  inflateEnd(context);
  return zerror;
}

More_Error
Ztcl_StreamFlush (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  int		zerror = Z_OK;

  
  if (! stream->finished)
    {
      zerror = stream->driver->flush(stream);
      Debug_PrintAvailableData(stream, "after flushing");
      Assert_ValidateOutputBuffer(stream);
    }
  return (Z_OK == zerror)? NULL : MakeErrorDescriptor(zerror);
}

static int
CompressFlush (Stream *stream)
{
  int		zerror;
  z_stream *	context = &(stream->zstream.context);
  

  debug("compression flushing");
  do
    {
      OutputBufferEnlarge(stream);
      zerror = deflate(context, Z_SYNC_FLUSH);
      Debug_PrintAvailableData(stream, "flushed block");
    }
  while ((Z_OK == zerror) && (context->avail_out));
  debug("error code %d, %d %d", zerror, context->avail_in, context->avail_out);
  if (Z_BUF_ERROR == zerror)
    {
      /*
	We reset  zerror because  it appears that  this is not  an error
	here:  it  just   means  that  data  was  flushed   as  much  as
	possible. This error is not  caused by "avail_out == 0", because
	we enlarged the buffer before the call to deflate().
      */
      zerror = Z_OK;
    }
  return zerror;
}

static int
DecompressFlush (Stream *stream)
{
  int		zerror, availableOutputBefore, availableOutputAfter;
  z_stream *	context = &(stream->zstream.context);


  debug("decompression flushing");
  do
    {
      OutputBufferEnlarge(stream);
      availableOutputBefore	= context->avail_out;
      zerror = inflate(context, Z_SYNC_FLUSH);
      availableOutputAfter	= context->avail_out;
      Debug_PrintAvailableData(stream, "flushed block");
    }
  while ((Z_OK == zerror) && (availableOutputBefore < availableOutputAfter));
  if (Z_BUF_ERROR == zerror)
    {
      /*
	We reset  zerror because  it appears that  this is not  an error
	here:  it  just   means  that  data  was  flushed   as  much  as
	possible. This error is not  caused by "avail_out == 0", because
	we enlarged the buffer before the call to deflate().
      */
      zerror = Z_OK;
    }
  return DecompressCheckEndOfStream(stream, zerror);
}

static int
DecompressCheckEndOfStream (Stream *stream, int zerror)
{
  if (Z_STREAM_END == zerror)
    {
      debug("finished decompression stream");
      inflateEnd(&(stream->zstream.context));
      stream->finished = 1;
      zerror = Z_OK;
    }
  return zerror;
}

void
Ztcl_StreamSetBufferSize (Ztcl_Stream token, int size)
{
  Stream *	stream = (Stream *) token;

  stream->output.size = size;
  if (OutputBufferIsAllocated(stream) &&
      (OutputBufferGetNumberOfUsedBytes(stream) < size))
    {
      More_BlockRealloc(stream->output.allocated, size);
    }
}
void
Ztcl_StreamSetBufferHysteresis (Ztcl_Stream token, int hysteresis)
{
  Stream *	stream = (Stream *) token;
  stream->output.hysteresis = hysteresis;
}
int
Ztcl_StreamGetBufferSize (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->output.allocated.len);
}
int
Ztcl_StreamGetBufferHysteresis (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->output.hysteresis);
}
int
Ztcl_StreamBufferAllocated (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return OutputBufferIsAllocated(stream);
}

int
Ztcl_StreamGetLevel (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->zstream.config.level);
}
int
Ztcl_StreamFinished (Ztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->finished);
}

static void
OutputBufferEnlarge (Stream *stream)
{
  int		hysteresis	= stream->output.hysteresis;
  z_stream *	context		= &(stream->zstream.context);
  More_Block *	outputBuffer	= &(stream->output.allocated);


  Debug_PrintAvailableData(stream, "before enlargement");
  if (! outputBuffer->len)
    {
      More_BlockAlloc(*outputBuffer, stream->output.size);
      context->next_out  = outputBuffer->ptr;
      context->avail_out = outputBuffer->len;
    }
  else if (ContextUnusedLength(context) < hysteresis)
    {
      size_t	newSize   = outputBuffer->len;
      ptrdiff_t	usedSpace = OutputBufferGetNumberOfUsedBytes(stream);

      newSize += (newSize % hysteresis) + hysteresis;
      More_BlockRealloc(*outputBuffer, newSize);
      context->next_out  = outputBuffer->ptr + usedSpace;
      context->avail_out = newSize - usedSpace;
    }
  Debug_PrintAvailableData(stream, "after  enlargement");
  Assert_ValidateOutputBuffer(stream);
}

static void
OutputBufferShrink (Stream *stream)
{
  int		hysteresis	= stream->output.hysteresis;
  z_stream *	context		= &(stream->zstream.context);
  More_Block *	outputBuffer	= &(stream->output.allocated);


  if (ContextUnusedLength(context) > (2*hysteresis))
    {
      int	usedSpace = OutputBufferGetNumberOfUsedBytes(stream);
      int	newSize   = usedSpace + usedSpace % hysteresis + hysteresis;

      More_BlockRealloc(*outputBuffer, newSize);
      context->next_out  = outputBuffer->ptr + usedSpace;
      context->avail_out = outputBuffer->len - usedSpace;
    }
  Debug_PrintAvailableData(stream, "shrinked buffer");
  Assert_ValidateOutputBuffer(stream);
}

static void *
StreamAlloc (void *opaque, uInt number, uInt size)
{
  return ((void *) ckalloc((unsigned) (number * size)));
}
static void
StreamFree (void *opaque, void *block)
{
  ckfree((char *) block);
}

static More_Error
RuntimeError (CONST char *errorInfo, int errorCode)
{
  More_Error error;


  error = More_ErrorNew();
  More_ErrorRuntimeStr(error, errorInfo);
  More_ErrorSetInt(error, errorCode);
  return error;
}

/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
