# automake.sh --
# 
# Part of: DevelStuff
# Contents: GNU automake replacement
# Date: Sat Jan  4, 2003
# 
# Abstract
# 
#	This script is the front-end to a collection of GNU m4 macros
#	stored in the file "automake.m4", that allows the automatic
#	rebuilding of a "Makefile.in" file from a template "Makefile.tm"
#	and a package specific template "Makefile.am".
#	It's a GNU BASH script.
#
#		$ bash ./.../automake.sh -v
#
# Copyright (c)  2003, 2004  Marco Maggi
# 
# This file  is free software you  can redistribute it  and/or modify it
# under the terms of the GNU  General Public License as published by the
# Free Software  Foundation; either version  2, or (at your  option) any
# later version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

set -e


#PAGE
## ------------------------------------------------------------
## Service variables required by MBFL.
## ------------------------------------------------------------

script_PROGNAME="automake.sh"
script_AUTHOR="Marco Maggi"
script_COPYRIGHT_YEARS="2003, 2004"

script_VERSION="1.0"
script_LICENSE=GPL

script_USAGE="usage: automake.sh [options]
DevelStuff automake script.
Options:
\t   --top-srcdir=DIR    selects the top directory of the source tree
\t   --srcdir=DIR        selects the current source directory
\t   --top-builddir=DIR  selects the top directory of the build tree
\t   --builddir=DIR      selects the build directory\n
\t-i --interactive       ask before doing dangerous operations
\t-f --force             do not ask before dangerous operations
\t   --encoded-args      decode arguments following this option
\t                       from the hex format (example: 414243 -> ABC)
\t-v --verbose           verbose execution
\t   --silent            silent execution
\t   --null              use the null character as terminator
\t   --debug             print debug messages
\t   --test              tests execution
\t   --version           print version informations and exit
\t   --version-only      print version number and exit
\t   --license           print license informations and exit
\t-h --help --usage      print usage informations and exit
"

#PAGE
## ------------------------------------------------------------
## Global variables.
## ------------------------------------------------------------

option_SRCDIR="${PWD}"
option_TOP_SRCDIR="${PWD}"
option_TOP_BUILDDIR="${PWD}"
option_BUILDDIR="${PWD}"

option_MAKEFILE_TEMPLATE="Makefile.tm"
option_MAKEFILE_INPUT="Makefile.am"
option_MAKEFILE_OUTPUT="Makefile.in"

#PAGE
## ------------------------------------------------------------
## Libraries.
## ------------------------------------------------------------

# libMBFL.m4 --
# 
# Part of: Marco's BASH Functions Library
# Contents: main library file
# Date: Fri Nov 28, 2003
# 
# Abstract
# 
# 
# 
# Copyright (c) 2003, 2004 Marco Maggi
# 
# This is free software; you  can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software  Foundation; either version  2.1 of the License,  or (at
# your option) any later version.
# 
# This library  is distributed in the  hope that it will  be useful, but
# WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# Lesser General Public License for more details.
# 
# You  should have  received a  copy of  the GNU  Lesser  General Public
# License along  with this library; if  not, write to  the Free Software
# Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
# USA
# 


function mbfl_decode_hex () {
local in="${1:?}"
local len="${#in}"
local i=0
while test $i -lt $len
do
echo -en "\\x${in:$i:2}"
i=$(($i + 2))
done
return 0
}
function mbfl_decode_oct () {
local in="${1:?}"
local len="${#in}"
local i=0
while test $i -lt $len
do
echo -en "\\${in:$i:3}"
i=$(($i + 3))
done
return 0
}

function mbfl_file_extension () {
local file="${1:?}"
local i="${#file}"
local ch=
while test $i -ge 0
do
ch="${file:$i:1}"
if test "$ch" = "/"
then
break
elif test "$ch" = "."
then
i=$(($i + 1))
echo "${file:$i}"
break
fi
i=$(($i - 1))
done
return 0
}
function mbfl_file_dirname () {
local file="${1:?}"
local i="${#file}"
while test $i -ge 0
do
if test "${file:$i:1}" = "/"
then
while test \( $i -gt 0 \) -a \( "${file:$i:1}" = "/" \)
do i=$(($i - 1))
done
if test $i = 0
then echo "${file:$i:1}"
else
i=$(($i + 1))
echo "${file:0:$i}"
fi
return 0
fi
i=$(($i - 1))
done
echo "."
return 0
}
function mbfl_file_normalise () {
local pathname="${1:?}"
local prefix="${2}"
if test -z "${prefix}"; then
prefix="${PWD}"
fi
if test "${pathname:0:1}" = "/"; then
echo "${pathname}"
elif test -d "${prefix}"; then
cd "${prefix}" >/dev/null
if test -d "${pathname}"; then
cd "${pathname}" >/dev/null
echo "${PWD}"
cd - >/dev/null
else
local tailname=$(mbfl_file_tail "${pathname}")
local dirname=$(mbfl_file_dirname "${pathname}")
if test -d "${dirname}"; then
cd "${dirname}" >/dev/null
echo "${PWD}/${tailname}"
cd - >/dev/null
else
echo "${prefix}/${pathname}"
fi
fi
else
echo "${prefix}/${pathname}"
fi
return 0
}
function mbfl_file_rootname () {
local file="${1:?}"
local i="${#file}"
if test $i = 1
then
echo "${file}"
return 0
fi
while test $i -ge 0
do
ch="${file:$i:1}"
if test "$ch" = "."
then
if test $i -gt 0
then
echo "${file:0:$i}"
break
else
echo "${file}"
fi
elif test "$ch" = "/"
then
echo "${file}"
break
fi
i=$(($i - 1))
done
return 0
}
function mbfl_file_split () {
local file="${1:?}"
echo -e "${file//\//\\n}"
return 0
}
function mbfl_file_tail () {
local file="${1:?}"
local i="${#file}"
while test $i -ge 0
do
ch="${file:$i:1}"
if test "$ch" = "/"
then
i=$(($i + 1))
echo "${file:$i}"
return 0
fi
i=$(($i - 1))
done
echo "${file}"
return 0
}
function mbfl_file_find_tmpdir () {
local TMPDIR="${1}"
if test -n "${TMPDIR}" -a -d "${TMPDIR}" -a -w "${TMPDIR}"
then
echo "${TMPDIR}"
return 0
fi
if test -n "${USER}"
then
TMPDIR="/tmp/${USER}"
if test -n "${TMPDIR}" -a -d "${TMPDIR}" -a -w "${TMPDIR}"
then
echo "${TMPDIR}"
return 0
fi
fi
TMPDIR=/tmp
if test -n "${TMPDIR}" -a -d "${TMPDIR}" -a -w "${TMPDIR}"
then
echo "${TMPDIR}"
return 0
fi
mbfl_message_error "cannot find usable value for TMPDIR"
return 1
}

ARGC=0
ARGC1=0
declare -a ARGV ARGV1
while test $# -gt 0
do
ARGV1[$ARGC1]="$1"
ARGC1=$(($ARGC1 + 1))
shift
done
function mbfl_getopts_parse () {
local OPT= OPTARG=
local arg= end=0 i=0
while test $i -lt $ARGC1
do
arg="${ARGV1[$i]}"
if test "$end" = 1
then
ARGV[$ARGC]="${arg}"
ARGC=$(($ARGC + 1))
elif test "${arg}" = "--"
then end=1
elif mbfl_getopts_isbrief "${arg}" OPT || \
mbfl_getopts_islong "${arg}" OPT
then mbfl_getopts_option "${OPT}"
elif \
mbfl_getopts_isbrief_with "${arg}" OPT OPTARG || \
mbfl_getopts_islong_with  "${arg}" OPT OPTARG
then mbfl_getopts_option_with "${OPT}" "${OPTARG}"
else
ARGV[$ARGC]="${arg}"
ARGC=$(($ARGC + 1))
fi
i=$(($i + 1))
done
return 0
}
function mbfl_getopts_option () {
local OPT="${1:?}"
local i=0
case "${OPT}" in
	encoded-args)
	    mbfl_option_ENCODED_ARGS="yes"
	    ;;
	v|verbose)
	    mbfl_option_VERBOSE="yes"
	    mbfl_message_VERBOSE="yes"
	    ;;
	silent)
	    mbfl_option_VERBOSE="no"
	    mbfl_message_VERBOSE="no"
	    ;;
	debug)
	    mbfl_option_DEBUG="yes"
	    mbfl_message_DEBUG="yes"
	    ;;
	test)
	    mbfl_program_TEST="yes"
	    ;;
	null)
	    mbfl_option_NULL="yes"
	    ;;
f|force)
	    mbfl_option_INTERACTIVE="no"
	    ;;
i|interactive)
	    mbfl_option_INTERACTIVE="yes"
	    ;;
	version)
	    echo -e "${mbfl_message_VERSION}"
	    exit 0
	    ;;
	version-only)
	    echo -e "${script_VERSION}"
	    exit 0
	    ;;
	license)
case "${script_LICENSE}" in
GPL)
echo -e "${mbfl_message_LICENSE_GPL}"
;;
LGPL)
echo -e "${mbfl_message_LICENSE_GPLL}"
;;
BSD)
echo -e "${mbfl_message_LICENSE_BSD}"
;;
*)
mbfl_message_error "unknown license: \"${script_LICENSE}\""
exit 2
;;
esac
	    exit 0
	    ;;
	h|help|usage)
	    echo -e "${script_USAGE}"
	    exit 0
	    ;;
	*)
if ! script_option "${OPT}"; then
mbfl_message_error "unknown option: \"${OPT}\""
exit 2
fi
	    ;;
esac
return 0
}
function mbfl_getopts_option_with () {
local OPT="${1:?}"
local OPTARG="${2:?}"
if test "${mbfl_option_ENCODED_ARGS}" = "yes"; then
OPTARG=$(mbfl_decode_hex "${OPTARG}")
fi
case "${OPT}" in
	*)
	    if ! script_option_with "${OPT}" "${OPTARG}"; then
mbfl_message_error "unknown option \"${OPT}\""
exit 2
fi
	    ;;
esac
return 0
}
function mbfl_getopts_islong () {
local arg="${1:?missing argument to mbfl_getopts_islong()}"
local optvar="${2}"
local len="${#arg}"
if test $len -lt 3
then return 1
fi
if test "${arg:0:1}" = "-" -a "${arg:1:1}" = "-"
then
local i=2
local ch=
while test $i -lt $len
do
ch="${arg:$i:1}"
if test \
\( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \) -a \
\( "$ch" \< 0 -o 9 \< "$ch" \) -a \
"$ch" != _ -a "$ch" != -
then return 1
fi
i=$(($i + 1))
done
if test -n "${optvar}"
then eval ${optvar}="${arg:2}"
fi
return 0
fi
return 1
}
function mbfl_getopts_islong_with () {
local arg="${1:?missing argument to mbfl_getopts_islong_with()}"
local optvar="${2}"
local valvar="${3}"
local len="${#arg}" str=
# The min length of a long option with is 5 (example: --o=1).
if test $len -lt 5 -o "${arg:0:1}" != "-" -o "${arg:1:1}" != "-"
then return 1
fi
local j=0
while test $j -lt $len
do
ch="${arg:$j:1}"
if test "$ch" = "="
then
if test $(($j + 1)) = $len
then return 1
fi
if mbfl_getopts_islong "${arg:0:$j}"
then
if test -n "${optvar}"
then eval ${optvar}="${arg:2:$(($j - 2))}"
fi
if test -n "${valvar}"
then
str=$(mbfl_getopts_quote_string "${arg:$(($j + 1))}")
eval "${valvar}=\"${str}\""
fi
return 0
else return 1
fi
fi
j=$(($j + 1))
done
return 1
}
function mbfl_getopts_isbrief () {
local arg="${1:?missing argument to mbfl_getopts_isbrief()}"
local optvar="${2}"
local ch="${arg:1:1}"
if test "${#arg}" = 2 -a "${arg:0:1}" = "-"
then
if test \( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \) -a \
\( "$ch" \< 0 -o 9 \< "$ch" \)
then return 1
else
if test -n "${optvar}"
then eval ${optvar}="${arg:1}"
fi
return 0
fi
else return 1
fi
}
function mbfl_getopts_isbrief_with () {
local arg="${1:?missing argument to mbfl_getopts_isbrief_with()}"
local optvar="${2}"
local valvar="${3}"
local ch="${arg:1:1}" str=
if test "${#arg}" -gt 2 -a "${arg:0:1}" = "-" -a "${arg:1:1}" != "-"
then
if test \( "$ch" \< A -a z \< "$ch" \) -a \
\( "$ch" \< 0 -a 9 \< "$ch" \)
then return 1
else
if test -n "${optvar}"
then eval ${optvar}="${arg:1:1}"
fi
if test -n "${valvar}"
then
str=$(mbfl_getopts_quote_string "${arg:2}")
eval "${valvar}=\"${str}\""
fi
return 0
fi
else return 1
fi
}
function mbfl_getopts_decode_hex () {
local i=0
while test $i -lt $ARGC
do
ARGV[$i]=$(mbfl_decode_hex "${ARGV[$i]}")
i=$(($i + 1))
done
return 0
}
function mbfl_wrong_num_args () {
local required="${1:?missing required number of args in mbfl_wrong_num_args()}"
local argc="${2:?missing given number of args in mbfl_wrong_num_args()}"
if test $required = $argc
then
return 0
else
mbfl_message_error "$required argument(s) required"
return 1
fi
}
function mbfl_getopts_quote_string () {
local STR="${1:?missing string to mbfl_getopts_quote_string()}"
local OUT=
local len="${#STR}" i=0 ch=

while test $i -lt $len
do
ch="${STR:$i:1}"
if test "$ch" = \\
then ch=\\\\
fi
OUT="${OUT}$ch"
i=$(($i + 1))
done
echo "$OUT"
return 0
}
function mbfl_argv_from_stdin () {
local file=
if test $ARGC -eq 0
then
	if test "${mbfl_option_NULL}" = "yes"; then
	    while read -d $'\x00' file
	      do
	      ARGV[$ARGC]="${file}"
	      ARGC=$(($ARGC + 1))
	    done
	else
	    while read file
	      do
	      ARGV[$ARGC]="${file}"
	      ARGC=$(($ARGC + 1))
	    done
	fi
fi
return 0
}
function mbfl_argv_all_files () {
local i=0
local file=
while test $i -lt $ARGC
do
	file="${ARGV[$i]}"
	file=$(mbfl_file_normalise "${file}")
	if test ! -f "${file}"
	then
	    mbfl_message_error "unexistent file \"${file}\""
	    return 1
	fi
	ARGV[$i]="${file}"
	i=$(($i + 1))
done
return 0
}

mbfl_message_PROGNAME="${0}"
mbfl_message_VERBOSE="no"
mbfl_message_DEBUGGING="no"
mbfl_message_STDERR="2"
function mbfl_message_set_progname () {
local PROGNAME="${1:?}"
mbfl_message_PROGNAME="${PROGNAME}"
return 0
}
function mbfl_message_set_verbosity () {
local VERBOSE="${1:?}"
mbfl_message_VERBOSE="${VERBOSE}"
return 0
}
function mbfl_message_set_debugging () {
local DEBUGGING="${1:?}"
mbfl_message_DEBUGGING="${DEBUGGING}"
return 0
}
function mbfl_message_set_stderr () {
local STDERR="${1:?}"
mbfl_message_STDERR="${STDERR}"
return 0
}
function mbfl_message_string () {
local MSG="${1:?missing argument to \"mbfl_message_string\"}"
echo -ne "${mbfl_message_PROGNAME}: ${MSG}" >&${mbfl_message_STDERR}
return 0
}
function mbfl_message_verbose () {
local MSG="${1:?missing argument to \"mbfl_message_verbose\"}"
if test "${mbfl_message_VERBOSE}" = 'yes'
then echo -ne "${mbfl_message_PROGNAME}: ${MSG}" >&${mbfl_message_STDERR}
fi
return 0
}
function mbfl_message_verbose_end () {
local MSG="${1:?missing argument to \"mbfl_message_verbose_end\"}"
if test "${mbfl_message_VERBOSE}" = 'yes'
then echo -ne "${MSG}" >&${mbfl_message_STDERR}
fi
return 0
}
function mbfl_message_debug () {
local MSG="${1:?missing argument to \"mbfl_message_debug\"}"
if test "${mbfl_message_DEBUGGING}" = 'yes'
then echo -ne "${mbfl_message_PROGNAME}: ${MSG}" >&${mbfl_message_STDERR}
fi
return 0
}
function mbfl_message_warning () {
local MSG="${1:?missing argument to \"mbfl_message_warning\"}"
echo -e "${mbfl_message_PROGNAME} warning: ${MSG}" >&${mbfl_message_STDERR}
return 0
}
function mbfl_message_error () {
local MSG="${1:?missing argument to \"mbfl_message_error\"}"
echo -e "${mbfl_message_PROGNAME} error: ${MSG}" >&${mbfl_message_STDERR}
return 0
}

function mbfl_program_check () {
local PROGRAMS=${*}
for item in ${PROGRAMS}
do
if test ! -x "${item}"
then
mbfl_message_error "cannot find executable \"${item}\""
return 1
fi
done
return 0
}
function mbfl_program_exec () {
if test "${mbfl_program_TEST}" = "yes"; then
echo "${@}" >&2
return 0
else
eval "${@}"
return $?
fi
}

declare -a MBFL_SIGNAL_MAP
declare -a MBFL_SIGNAL_HANDLERS
declare -a MBFL_SIGNAL_REGISTRY
function mbfl_signal_map_signame_to_signum () {
local SIGSPEC="${1:?}"
declare -i signum=0
while test -n "${MBFL_SIGNAL_MAP[${signum}]}"
do
if test "${MBFL_SIGNAL_MAP[${signum}]}" = "${SIGSPEC}"
then break
fi
signum=${signum}+1
done
MBFL_SIGNAL_MAP[$signum]="${SIGSPEC}"
echo ${signum}
return 0
}
function mbfl_signal_register_handler () {
local HANDLER="${1:?}"
declare -i hannum
hannum=$((${#MBFL_SIGNAL_HANDLERS[*]} + 1))
MBFL_SIGNAL_HANDLERS[${hannum}]="${HANDLER}"
return 0
}
function mbfl_signal_attach () {
local SIGSPEC="${1:?}"
local HANDLER="${2:?}"
declare -i signum=0 hannum=0
# Append  the handler  number  to the  list  of registered  handlers
# associated to the signal number.
MBFL_SIGNAL_REGISTRY[$signum]="${MBFL_SIGNAL_REGISTRY[$signum]} $hannum"
trap "mbfl_signal_handler ${signum}" ${SIGSPEC}
echo $hannum
return 0
}
function mbfl_signal_detach () {
local hannum="${1:?}"
if test -n "${MBFL_SIGNAL_HANDLERS[${hannum}]}"
then
MBFL_SIGNAL_HANDLERS[${hannum}]=
return 0
else
mbfl_message_error "signal handler not found when detaching"
return 1
fi
}
function mbfl_signal_handler () {
local signum="${1:?}"
local HANDLER=
declare -i signum=0 hannum=
for hannum in ${MBFL_SIGNAL_REGISTRY[$signum]}
do
HANDLER="${MBFL_SIGNAL_HANDLERS[$hannum]}"
if test -n "${HANDLER}"
then eval ${HANDLER}
fi
done
return 0
}
function mbfl_signal_inspect () {
local SIGSPEC=
local HANDLER=
declare -i signum=0 hannum=
mbfl_message_string "Inspecting signal table:\n"
while test $signum -lt ${#MBFL_SIGNAL_MAP[*]}
do
SIGSPEC="${MBFL_SIGNAL_MAP[${signum}]}"
if test -z "${SIGSPEC}"
then break
fi
for handnum in ${MBFL_SIGNAL_REGISTRY[${signum}]}
do
HANDLER="${MBFL_SIGNAL_HANDLERS[${handnum}]}"
if test -z "${HANDLER}"
then break
fi
echo "Signal \"${SIGSPEC}\": ${HANDLER} "
done
signum=${signum}+1
done
mbfl_message_string "End inspection.\n"
return 0
}

function mbfl_string_chars () {
local string="${1:?}"
local i=0
while test $i -lt "${#string}"
do
echo ${string:$i:1}
i=$(($i + 1))
done
}
function mbfl_string_first () {
local string="${1:?}"
local char="${2:?}"
local begin="${3:-0}"
local i=$begin
while test "$i" -lt "${#string}"
do
if test "${string:$i:1}" = "$char"
then
echo $i
return 0
fi
i=$(($i+1))
done
return 0
}
function mbfl_string_index () {
local string="${1:?}"
local index="${2:?}"
echo "${string:$index:1}"
}
function mbfl_string_is () {
local class="${1:?null class to mbfl_string_is}"
local string="${2}"
local ch=
local flag=0
test "${#string}" -eq 0 && return 1
case "$class" in
alpha)
{
local i=0
while test $i -lt "${#string}"
do
ch="${string:$i:1}"
if test \
\( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \)
then return 1
fi
i=$(($i + 1))
done
} || flag=1
;;
alnum)
{
local i=0
while test $i -lt "${#string}"
do
ch="${string:$i:1}"
if test \
\( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \) -a \
\( "$ch" \< 0 -o 9 \< "$ch" \)
then return 1
fi
i=$(($i + 1))
done
} || flag=1
;;
digit)
{
local i=0
while test $i -lt "${#string}"
do
ch="${string:$i:1}"
if test \( "$ch" \< 0 -o 9 \< "$ch" \)
then return 1
fi
i=$(($i + 1))
done
} || flag=1
;;
noblank)
local i=0
local len="${#string}"
while test $i -lt $len
do
ch="${string:$i:1}"
if test "$ch" = " "   -o "$ch" = $'\n' -o \
"$ch" = $'\r' -o "$ch" = $'\t' -o \
"$ch" = $'\f'
then return 1
fi
i=$(($i + 1))
done
return 0
;;
name)
local i=1
local len="${#string}"
local ch=
if test $len -eq 0
then return 1
else
ch="${string:0:1}"
if test \( "$ch" != _ \) -a \
\( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \)
then return 1
fi
while test $i -lt $len
do
ch="${string:$i:1}"
if test \( "$ch" != _ \) -a \
\( "$ch" \< A -o Z \< "$ch" \) -a \
\( "$ch" \< a -o z \< "$ch" \) -a \
\( "$ch" \< 0 -o 9 \< "$ch" \)
then return 1
fi
i=$(($i + 1))
done
fi
return 0
;;
esac
return $flag
}
function mbfl_string_last () {
local string="${1:?}"
local char="${2:?}"
local begin="${3:-${#string}}"
local i=$begin
while test "$i" -gt -1
do
if test "${string:$i:1}" = "$char"
then
echo $i
return 0
fi
i=$(($i - 1))
done
return 0
}
function mbfl_string_range () {
local string="${1:?}"
local begin="${2:?}"
local end="${3}"
if test -z "$end" -o "$end" = "end"
then echo "${string:$begin}"
else echo "${string:$begin:$end}"
fi
}
function mbfl_string_split () {
local string="${1:?}"
local chars="${2:?}"
local i=0
local j=0
local sep=
while test $i -lt "${#chars}"
do
sep="${chars:$i:1}"
j=0
while test $j -lt "${#string}"
do
string="${string//${sep}/\\n}"
j=$(($j + 1))
done
i=$(($i + 1))
done
echo -e "$string"
return 0
}

function mbfl_dialog_yes_or_no () {
local STRING="${1:?}"
local ANS=
read -e -p "${STRING}? (yes/no) " ANS
while test "$ANS" != 'yes' -a "$ANS" != 'no'
do
echo 'Please answer yes or no.'
read -e -p "install?(yes/no) " ANS
done
if test "$ANS" = yes; then
return 0
else
return 1
fi
}

mbfl_option_VERBOSE="no"
mbfl_option_DEBUG="no"
mbfl_option_TEST="no"
mbfl_option_INTERACTIVE="no"
mbfl_option_ENCODED_ARGS="no"
mbfl_option_TMPDIR="${TMPDIR:-/tmp}"
mbfl_option_ACTION=
mbfl_ORG_PWD="${PWD}"
mbfl_message_LICENSE_GPL="${script_PROGNAME} version ${script_VERSION}
Written by ${script_AUTHOR}.\n
Copyright (C) ${script_COPYRIGHT_YEARS} by ${script_AUTHOR}.\n
This file  is free software you  can redistribute it  and/or modify it
under the terms of the GNU  General Public License as published by the
Free Software  Foundation; either version  2, or (at your  option) any
later version.\n
This  file is  distributed in  the hope  that it  will be  useful, but
WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
General Public License for more details.\n
You  should have received  a copy  of the  GNU General  Public License
along with this file; see the file COPYING.  If not, write to the Free
Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
02111-1307, USA.
"
mbfl_message_LICENSE_LGPL="${script_PROGNAME} version ${script_VERSION}
Written by ${script_AUTHOR}.\n
Copyright (C) ${script_COPYRIGHT_YEARS} by ${script_AUTHOR}.\n
This is free software; you  can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software  Foundation; either version  2.1 of the License,  or (at
your option) any later version.\n
This library  is distributed in the  hope that it will  be useful, but
WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
Lesser General Public License for more details.\n
You  should have  received a  copy of  the GNU  Lesser  General Public
License along  with this library; if  not, write to  the Free Software
Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
USA.
"
mbfl_message_LICENSE_BSD="${script_PROGNAME} version ${script_VERSION}
Written by ${script_AUTHOR}.\n
Copyright (C) ${script_COPYRIGHT_YEARS} by ${script_AUTHOR}.\n
The author  hereby grant permission to use,  copy, modify, distribute,
and  license this  software  and its  documentation  for any  purpose,
provided that  existing copyright notices  are retained in  all copies
and that  this notice  is included verbatim  in any  distributions. No
written agreement, license, or royalty  fee is required for any of the
authorized uses.  Modifications to this software may be copyrighted by
their authors and need not  follow the licensing terms described here,
provided that the new terms are clearly indicated on the first page of
each file where they apply.\n
IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
POSSIBILITY OF SUCH DAMAGE.\n
THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN \"AS  IS\" BASIS,
AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
"
mbfl_message_VERSION="${script_PROGNAME} version ${script_VERSION}
Written by ${script_AUTHOR}.\n
Copyright (C) ${script_COPYRIGHT_YEARS} by ${script_AUTHOR}.\n
This is  free software; see the  source or use the  --license option for
copying conditions.  There is NO warranty; not  even for MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.
"
function mbfl_main () {
local exit_code=0
mbfl_message_set_progname "${script_PROGNAME}"
mbfl_getopts_parse
if test "${mbfl_option_ENCODED_ARGS}" = "yes"; then
	mbfl_getopts_decode_hex
fi
mbfl_message_set_verbosity "${mbfl_option_VERBOSE}"
mbfl_message_set_debugging "${mbfl_option_DEBUG}"
if test "${mbfl_option_TEST}" = "yes"; then
	mbfl_message_string "test execution\n"
fi
script_begin || exit $?
if test -n "${mbfl_option_ACTION}"; then
	script_action_"${mbfl_option_ACTION}"
	exit_code=$?
fi
script_end || exit $?
exit $exit_code
}


### end of file
# Local Variables:
# mode: sh
# page-delimiter: "^#PAGE$"
# End:


#PAGE
function script_begin () {
    return 0
}
function script_end () {
    return 0
}
#PAGE
function script_action_process_template () {
    local M4=$(type -path m4)
    local M4FLAGS="--prefix-builtins"

    local top_srcdir="${option_TOP_SRCDIR}"
    local srcdir="${option_SRCDIR}"
    local builddir="${option_BUILDDIR}"

    local INPUT="${srcdir}/${option_MAKEFILE_INPUT}"
    local OUTPUT="${srcdir}/${option_MAKEFILE_OUTPUT}"
    local TEMPLATE="${srcdir}/${option_MAKEFILE_TEMPLATE}"

    local item= dummy=


    # Check the availability of GNU m4.

    if ! mbfl_program_check "${M4}"; then
	return 1
    fi
    if ! dummy=$("${M4}" --version); then
	mbfl_message_error "cannot find usable GNU m4"
	return 1
    fi
    set -- ${dummy}
    if test "${1}" != GNU; then
	mbfl_message_error "cannot find usable GNU m4"
	return 1
    fi

    # Find input template.

    TEMPLATE="${PWD}/Makefile.ds"
    if test ! -f "${TEMPLATE}"; then
	mbfl_message_error "unexistent template file (${TEMPLATE})"
	return 1
    fi

    # Find macros file.

    for dummy in 1; do
	for CONFIGDIR in gnuconfig tclconfig config; do
	    MACROFILE="${PWD}/${CONFIGDIR}/automake.m4"
	    if test -f "${MACROFILE}"; then
		break 2
	    fi
	done
	mbfl_message_error "unexistent macros file (automake.m4)"
	return 1
    done

    # Source dir for test files.
    for ds_tmp in src/tests tests src/test test . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_TEST_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source test directory: ${DS_TEST_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_TEST_SRCDIR_MACRO=${DS_TEST_SRCDIR_MACRO}"
    
    # Main test file for TCL tests.
    for ds_tmp in ${DS_TEST_SRCDIR_MACRO}; do
	ds_tmp="${ds_tmp}/all.tcl"
	if test -f "${ds_tmp}"; then
	    DS_TCLTESTMAIN_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "TCL main test file: ${DS_TCLTESTMAIN_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_TCLTESTMAIN_MACRO=${DS_TCLTESTMAIN_MACRO}"

    # Source dir for example files.
    for ds_tmp in src/examples examples src/example example . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_EXAMPLE_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source example directory: ${DS_EXAMPLE_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_EXAMPLE_SRCDIR_MACRO=${DS_EXAMPLE_SRCDIR_MACRO}"
    
    # Source dir for documentation files.
    for ds_tmp in src/docs docs src/doc doc . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_DOC_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source doc directory: ${DS_DOC_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_DOC_SRCDIR_MACRO=${DS_DOC_SRCDIR_MACRO}"
    
    # Generic source dir.
    for ds_tmp in src/generic generic . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_GENERIC_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source generic directory: ${DS_GENERIC_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_GENERIC_SRCDIR_MACRO=${DS_GENERIC_SRCDIR_MACRO}"
    
    # Mac source dir.
    for ds_tmp in src/mac mac . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_MAC_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source mac directory: ${DS_MAC_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_MAC_SRCDIR_MACRO=${DS_MAC_SRCDIR_MACRO}"
    
    # Win source dir.
    for ds_tmp in src/win win . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_WIN_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source win directory: ${DS_WIN_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_WIN_SRCDIR_MACRO=${DS_WIN_SRCDIR_MACRO}"
    
    # Unix source dir.
    for ds_tmp in src/unix unix . ; do
	ds_tmp="${top_srcdir}/${ds_tmp}"
	if test -d "${ds_tmp}"; then
	    DS_UNIX_SRCDIR_MACRO="${ds_tmp}"
	    break
	fi
    done
    mbfl_message_verbose "source unix directory: ${DS_UNIX_SRCDIR_MACRO}\n"
    M4FLAGS="${M4FLAGS} --define=DS_UNIX_SRCDIR_MACRO=${DS_UNIX_SRCDIR_MACRO}"


    # Create the output template.
    
    mbfl_message_verbose "creating template...\n"
    if ! "${M4}" ${M4FLAGS} "${MACROFILE}" "${TEMPLATE}" >./Makefile.in; then
	return 1
    fi
    mbfl_message_verbose "done.\n"

    return 0
}
#PAGE
function script_option () {
    local OPTNAME="${1}"


    case "${OPTNAME}" in
	*)
	    return 2
	    ;;
    esac
    return 0
}
#PAGE
function script_option_with () {
    local OPT="${1:?}"
    local OPTARG="${2:?}"

    case "$OPT" in
	top-srcdir)
	    option_TOP_SRCDIR="${OPTARG}"
	    ;;
	top-builddir)
	    option_TOP_BUILDDIR="${OPTARG}"
	    ;;
	srcdir)
	    option_SRCDIR="${OPTARG}"
	    ;;
	builddir)
	    option_BUILDDIR="${OPTARG}"
	    ;;

	*)
	    mbfl_message_error "unknown option \"${arg}\""
	    echo -e "${message_USAGE}" >&2
	    exit 2
	    ;;
    esac
    return 0
}
#PAGE
## ------------------------------------------------------------
## Main script.
## ------------------------------------------------------------

mbfl_option_ACTION=process_template
mbfl_main

### end of file
# Local Variables:
# mode: sh
# page-delimiter: "^#PAGE$"
# End:
