#  mktip55.tcl --
# 
# Part of: DevelStuff
# Contents: builds TIP55 compliant archives
# Date: Mon May 17, 2004
# 
# Abstract
# 
# 
# 
# Copyright (c) 2004 Marco Maggi
# 
# This is free  software you can redistribute it  and/or modify it under
# the terms of  the GNU General Public License as  published by the Free
# Software Foundation; either  version 2, or (at your  option) any later
# version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;  without   even  the   implied  warranty   of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

#PAGE
## ------------------------------------------------------------
## Global variables.
## ------------------------------------------------------------

set zip [auto_execok zip]
set architecture "i586-pc-linux-gnu"

#PAGE
proc main { argc argv } {
    global	description builddir architecture zip
    set exit_code 0
    
    # We assume that the script is executed from the build directory.

    set builddir [pwd]

    if { [catch {
	description_read
	set description(Architecture) $architecture
    
	set name [format "%s%s" $description(Identifier) $description(Version)]
	set zip_tmp			[file join $builddir tmp]
	set zip_root_dir		[file join $zip_tmp $name]
	set zip_file_name		[format "%s.zip" $name]
	set zip_full_file_name		[file join $builddir $zip_file_name]
	set zip_architecture_dir	[file join $zip_root_dir $architecture]
	set zip_doc_dir			[file join $zip_root_dir doc]

	if { [catch {
	    if { [file exists $zip_root_dir] } {
		message_error [format "removing old %s" $zip_root_dir]
		file delete -force -- $zip_root_dir
	    }
	    if { [file exists $zip_full_file_name] } {
		message_error [format "removing old %s" $zip_full_file_name]
		file delete -force -- $zip_full_file_name
	    }
	    file mkdir $zip_root_dir
	    file mkdir $zip_doc_dir
	    file mkdir $zip_architecture_dir
	} res] } {
	    error [format "building directory structure: %s" $res]
	}

	if { [catch {
	    zip_write_description [file join $zip_root_dir DESCRIPTION.txt]

	    # The version number read from the description file may have
	    # an "a"  or a  "b" between the  minor number and  the patch
	    # level. This will not be  in the library name, so we remove
	    # it.
	    set libversion [string map {a . b .} $description(Version)]
	
	    # Install the library file.
	    set libfile \
		[format "lib%s%s.so" $description(Identifier) $libversion]
	    set zip_libfile [file join $zip_architecture_dir $libfile]
	    message_verbose [format "%s -> %s" $libfile $zip_libfile]
	    file copy $libfile $zip_libfile

	    # Install the stub library file.
	    set stublibfile \
		[format "lib%sstub%s.a" $description(Identifier) $libversion]
	    set zip_stublibfile [file join $zip_architecture_dir $stublibfile]
	    message_verbose [format "%s -> %s" $stublibfile $zip_stublibfile]
	    file copy $stublibfile $zip_stublibfile
	    
	    # Install the documentation files.
	    foreach docfile [glob -nocomplain -- *.html *.info*] {
		set destfile [file join $zip_doc_dir $docfile]
		message_verbose [format "%s -> %s" $docfile $destfile]
		file copy $docfile $destfile
	    }
	} res] } {
	    error [format "error installing files: %s" $res]
	}

	if { [catch {
	    cd $zip_tmp
	    if { ! [file isdirectory $name] } {
		puts stderr "source directory for archive not found"
		exit 1
	    }
	    
	    # Create the archive and verify it.
	    file delete -force -- $zip_full_file_name
	    exec $zip -v -9 -r $zip_full_file_name $name >@stdout 2>@stderr
	    exec $zip -v -T    $zip_full_file_name >@stdout 2>@stderr
	} res] } {
	    error [format "error compressing file: %s" $res]
	}
    } res] } {
	message_error $res
	set exit_code 1
    }

    zip_clean $zip_tmp
    exit $exit_code
}
#PAGE
proc description_read { } {
    global	builddir description
    set fileName [file join $builddir meta DESCRIPTION.txt]


    # Load file.

    if { ! [file isfile $fileName] } {
	puts stderr "missing description file"
	exit 1
    }
    if { [catch {
	set id [open $fileName RDONLY]
	set buffer [read $id]
	close $id
    } res] } {
	puts stderr "error reading description file: $res"
	exit 1
    }

    # Parse fields.
    
    array set description {}
    set lines [split $buffer \n]

    for {set i 0} {$i < [llength $lines] } {incr i} {
	set line [lindex $lines $i]

	if { [string length $line] == 0} {
	    continue
	}
	if { ! [regexp -- {^([a-zA-Z]+):\s+(.+)$} $line dummy header value] } {
	    return -code error \
		[format "parsing description file bad line: \"%s\"" $line]
	}
	if { ! [string equal $header "Description"] } {
	    set description($header) $value
	} else {
	    set line [lindex $lines [incr i]]

	    while {
		($i < [llength $lines]) &&
		([regexp -- {^\s+.*$} $line])
	    } {
		append value \n $line
		set line [lindex $lines [incr i]]
	    }
	    set description($header) $value
	    incr i -1
	}
    }

    # Validate fields.

    if {
	(! [info exists description(Identifier)]) ||
	(! [info exists description(Version)])
    } {
	return -code error \
	    "missing identifier or version in description file"
    }

    return
}
#PAGE
proc zip_write_description { file } {
    global	description


    message_verbose "writing description $file"

    set buffer {}
    foreach {key val} [array get description] {
	append buffer [format "%s: %s" $key $val] \n
    }

    if { [catch {
	set id [open $file {CREAT EXCL WRONLY}]
	puts -nonewline $id $buffer
	close $id
    } res] } {
	return -code error [format "writing description file: %s" $res]
    }
    return
}
#PAGE
proc zip_clean { zip_tmp } {
    if { [catch {file delete -force -- $zip_tmp} res] } {
	message_warning [format "cleaning tmp dir: %s" $res]
    }
}
#PAGE
proc message_error { string } {
    puts stderr [format "mktip55.tcl error: %s" $string]
}
proc message_warning { string } {
    puts stderr [format "mktip55.tcl warning: %s" $string]
}
proc message_verbose { string } {
    puts stderr [format "mktip55.tcl: %s" $string]
}
#page

main $argc $argv

### end of file
